/*
*********************************************************************************************************
*                                              uC/OS-III
*                                        The Real-Time Kernel
*
*                          (c) Copyright 2004-2009; Micrium, Inc.; Weston, FL               
*
*               All rights reserved.  Protected by international copyright laws.
*
*               uC/OS-II is provided in source form for FREE evaluation, for educational
*               use or peaceful research.  If you plan on using uC/OS-II in a commercial    
*               product you need to contact Micrium to properly license its use in your     
*               product.  We provide ALL the source code for your convenience and to        
*               help you experience uC/OS-II.  The fact that the source code is provided
*               does NOT mean that you can use it without paying a licensing fee.
*
*               Knowledge of the source code may NOT be used to develop a similar product.
*
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                       APPLICATION CONFIGURATION
*
*                                  SMART FUSION EVALUATION KIT BOARD
*
* Filename      : app_cfg.h
* Version       : V1.00
* Programmer(s) : FT
*********************************************************************************************************
*/

#ifndef  APP_CFG_MODULE_PRESENT
#define  APP_CFG_MODULE_PRESENT


/*
*********************************************************************************************************
*                                            TASKS PRIORITIES
*********************************************************************************************************
*/

#define  APP_CFG_TASK_START_PRIO                   	0
#define  APP_CFG_TASK_IO_PRIO                   	5
#define  APP_CFG_TASK_SOV_PRIO                   	6
#define  APP_CFG_TASK_UART0TX_PRIO                  7
#define  APP_CFG_TASK_PWM_PRIO                  	9
#define  APP_CFG_TASK_UART2TX_PRIO                  10
#define	 MB_OS_CFG_RX_TASK_PRIO						12

/*
*********************************************************************************************************
*                                            TASKS ID
*********************************************************************************************************
*/
#define	 MB_OS_CFG_RX_TASK_ID						MB_OS_CFG_RX_TASK_PRIO

/*
*********************************************************************************************************
*                                              STACK SIZES
*                            Size of the task stacks (# of OS_STK entries)
*********************************************************************************************************
*/

#define  APP_CFG_TASK_START_STK_SIZE            	512
#define  APP_CFG_TASK_SOV_STK_SIZE              	512
#define  APP_CFG_TASK_IO_STK_SIZE              		512
#define  APP_CFG_TASK_UART0TX_STK_SIZE              512
#define  APP_CFG_TASK_PWM_STK_SIZE              	512
#define  APP_CFG_TASK_UART2TX_STK_SIZE              512
#define  MB_OS_CFG_RX_TASK_STK_SIZE					512

/*
*********************************************************************************************************
*                                             uC/LIB CONFIGURATION
*********************************************************************************************************
*/

#define  LIB_MEM_CFG_OPTIMIZE_ASM_EN             DEF_DISABLED
#define  LIB_MEM_CFG_ARG_CHK_EXT_EN              DEF_ENABLED
#define  LIB_MEM_CFG_ALLOC_EN                    DEF_DISABLED
#define  LIB_MEM_CFG_HEAP_SIZE                         1024u

/*
*********************************************************************************************************
*                                     TRACE / DEBUG CONFIGURATION
*********************************************************************************************************
*/

#define  TRACE_LEVEL_OFF                                   0
#define  TRACE_LEVEL_INFO                                  1
#define  TRACE_LEVEL_DBG                                   2

#define  APP_CFG_TRACE_LEVEL                    TRACE_LEVEL_INFO
#define  APP_CFG_TRACE                          printf

#define  APP_TRACE_INFO(x)                     ((APP_CFG_TRACE_LEVEL >= TRACE_LEVEL_INFO)  ? (void)(APP_CFG_TRACE x) : (void)0)
#define  APP_TRACE_DBG(x)                      ((APP_CFG_TRACE_LEVEL >= TRACE_LEVEL_DBG)   ? (void)(APP_CFG_TRACE x) : (void)0)

/*
*********************************************************************************************************
*                                              UART CONFIGURATION
*********************************************************************************************************
*/

#define RX_BUFF_SIZE    5		/* UART receive buffer size. */

/*
*********************************************************************************************************
*                                              PWM CONFIGURATION
*********************************************************************************************************
*/

/******************************************************************************
 * PWM prescale and period configuration values.
 *****************************************************************************/
#define PWM_PRESCALE    8
#define PWM_PERIOD      100

/*
*********************************************************************************************************
*                                        BSP CONFIGURATION
*
* Note(s) : (1) BSP_CFG_SER_COMM_SEL defines the UART to be used to output the system state:
*
*                   BSP_SER_COMM_UART_00           UART #0  is selected to output the system state
*                   BSP_SER_COMM_UART_01           UART #1  is selected to output the system state
*                   BSP_SER_COMM_UART_DBG          UART DBG is selected to output the system state
*
*********************************************************************************************************
*/

#define  BSP_CFG_SER_COMM_SEL                      BSP_SER_COMM_UART_DBG

/*
*********************************************************************************************************
*                                       MODBUS BSP & APP CONFIGURATION
*********************************************************************************************************
*/

#define MODBUS_ACTIVE							1
#define MODBUS_SLAVE_MODE_ENABLE_DEBUG			0

#define MB_BSP_PORT_UART_00						0x01
#define MB_BSP_PORT_UART_01						0x02
#define MB_BSP_PORT_UART_02						0x03

#define  MODBUS_STOP_BIT_1                         1
#define  MODBUS_STOP_BIT_2                         2

#define  MB_BSP_CFG_UART_00_MODE                  MB_BSP_UART_RS232_MODE
#define  MB_BSP_CFG_UART_01_MODE                  MB_BSP_UART_RS232_MODE
#define  MB_BSP_CFG_UART_02_MODE                  MB_BSP_UART_RS232_MODE

#define  MS_BSP_CFG_TMR                           MB_BSP_TMR_0

#define  APP_CFG_MODBUS_CH1_NODE_ADDR                     2
#if MODBUS_SLAVE_MODE_ENABLE_DEBUG
#define  APP_CFG_MODBUS_CH1_MODE                  MODBUS_SLAVE
#define  APP_CFG_MODBUS_CH1_TIMEOUT                     0
#else
#define  APP_CFG_MODBUS_CH1_MODE                  MODBUS_MASTER
#define  APP_CFG_MODBUS_CH1_TIMEOUT                     5
#endif

#define  APP_CFG_MODBUS_CH1_COMM_MODE             MODBUS_MODE_RTU
#define  APP_CFG_MODBUS_CH1_PORT                  MB_BSP_UART_00
#define  APP_CFG_MODBUS_CH1_BAUD_RATE                  57600
#define  APP_CFG_MODBUS_CH1_NBR_BITS              MSS_UART_DATA_8_BITS
#define  APP_CFG_MODBUS_CH1_NBR_PARITY            MSS_UART_NO_PARITY
#define  APP_CFG_MODBUS_CH1_STOP_BITS             MSS_UART_ONE_STOP_BIT
#define  APP_CFG_MODBUS_CH1_WR_EN                 MODBUS_WR_EN

#define  APP_CFG_MODBUS_CH2_NODE_ADDR                     2
#define  APP_CFG_MODBUS_CH2_MODE                  MODBUS_MASTER
#define  APP_CFG_MODBUS_CH2_TIMEOUT                       0
#define  APP_CFG_MODBUS_CH2_COMM_MODE             MODBUS_MODE_ASCII
#define  APP_CFG_MODBUS_CH2_PORT                  MB_BSP_UART_01
#define  APP_CFG_MODBUS_CH2_BAUD_RATE               115200
#define  APP_CFG_MODBUS_CH2_NBR_BITS              MSS_UART_DATA_8_BITS
#define  APP_CFG_MODBUS_CH2_NBR_PARITY            MSS_UART_NO_PARITY
#define  APP_CFG_MODBUS_CH2_STOP_BITS             MSS_UART_ONE_STOP_BIT
#define  APP_CFG_MODBUS_CH2_WR_EN                 MODBUS_WR_EN

#define  APP_CFG_MODBUS_CH3_NODE_ADDR                     2
#define  APP_CFG_MODBUS_CH3_MODE                  MODBUS_MASTER
#define  APP_CFG_MODBUS_CH3_TIMEOUT                       0
#define  APP_CFG_MODBUS_CH3_COMM_MODE             MODBUS_MODE_ASCII
#define  APP_CFG_MODBUS_CH3_PORT                  MB_BSP_UART_02
#define  APP_CFG_MODBUS_CH3_BAUD_RATE               115200
#define  APP_CFG_MODBUS_CH3_NBR_BITS                     8
#define  APP_CFG_MODBUS_CH3_NBR_PARITY            MODBUS_PARITY_NONE
#define  APP_CFG_MODBUS_CH3_STOP_BITS             MODBUS_STOP_BIT_1
#define  APP_CFG_MODBUS_CH3_WR_EN                 MODBUS_WR_EN

#define OS_TIMEOUT 									OS_ERR_TIMEOUT
#define OS_NO_ERR  									OS_ERR_NONE

#define MB_F06_SET_HI_PRESS_ADDR				  34
#define MB_F06_SET_HI_PRESS_VALUE				  2000


#endif
