/*
*********************************************************************************************************
*                                     MICRIUM BOARD SUPPORT SUPPORT
*
*                          (c) Copyright 2003-2009; Micrium, Inc.; Weston, FL
*
*               All rights reserved.  Protected by international copyright laws.
*               Knowledge of the source code may NOT be used to develop a similar product.
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*
*                                      BOARD SUPPORT PACKAGE (BSP)
*
*                                  SMART FUSION EVALUATION KIT BOARD
*
* Filename      :bsp.c
* Version       :V1.00
* Programmer(s) :FT
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                             INCLUDE FILES
*********************************************************************************************************
*/

#define   BSP_MODULE
#include  <includes.h>


/*
*********************************************************************************************************
*                                           LOCAL CONSTANTS
*********************************************************************************************************
*/
const uint8_t message_ClearScreen[]		= "\033[2J";

/*
*********************************************************************************************************
*                                          LOCAL DATA TYPES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            LOCAL TABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/
/******************************************************************************
 * CoreUARTapb instance data.
 *****************************************************************************/
UART_instance_t g_uart0, g_uart2;

/******************************************************************************
 * CorePWM instance data.
 *****************************************************************************/

uint32_t duty_cycle = 1;

/*
*********************************************************************************************************
*                                               MACRO'S
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                      LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/

static  CPU_BOOLEAN  BSP_CPU_Init  (void);
void BSP_INIT_CNTR();


/*
*********************************************************************************************************
*                                     LOCAL CONFIGURATION ERRORS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*********************************************************************************************************
**                                         GLOBAL FUNCTIONS
*********************************************************************************************************
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                             BSP_LowLevelInit()
*
* Description : System Low Level Initialization.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : Startup Code.
* 
* Note(s)     : (1) Perform low level initialization before callin main().
*********************************************************************************************************
*/

void  BSP_LowLevelInit (void)
{

}


/*
*********************************************************************************************************
*                                             BSP_PreInit()
*
* Description : System Pre-Initialization. Initializes all peripherals that don't require OS services (LEDs, PBs)
*               or modules than need to be initialized before the OS (External memories).
*
* Argument(s) : main().
*
* Return(s)   : none.
*
* Note(s)     : (1) The following configuration is assumed on the Evaluation Kit:
* 
*                       +-------------|------------|--------------------+--------------+
*                       |    A2F200   |   A2F200   |  EVALUATION KIT    |  BREADBOARD  |
*                       |  PERIPHERAL |  PIN NAME  |       SIGNAL       |      v02     |
*                       +-------------|------------|--------------------+--------------|
*                       |   GPO_00    |    B19     |       LED1_N       |  SOV Status  |
*                       |   GPO_01    |    J06     |       LED2_N       |  Drive FM    |
*                       |   GPO_02    |    C19     |       LED3_N       |              |
*                       |   GPO_03    |    H17     |       LED4_N       |              |
*                       |   GPO_04    |    H20     |       LED5_N       |              |
*                       |   GPO_05    |    C21     |       LED6_N       |              |
*                       |   GPO_06    |    D21     |       LED7_N       |              |
*                       |   GPO_07    |    G21     |       SWITCH1      |              |
*                       |   GPO_08    |    G21     |       SWITCH2      |              |
*                       |   GPO_09    |            |                    |              |
*                       |   GPO_10    |            |                    |              |
*                       |   GPO_11    |            |                    |              |
*                       |   GPO_12    |    AA2     |                    |SOV: Diag Ena |
*                       |   GPO_13    |    AB2     |                    |SOV: Norm Ena |
*                       |   GPO_14    |    G19     |                    |   SWITCH1    |
*                       |   GPO_15    |    G20     |                    |   SWITCH2    |
*                       +-------------+------------+--------------------+--------------+
*
*               (2) This initialization code is not needed if the Libero IDE is used to configured the MSS.
*                   and the system boot code is executed first.
*********************************************************************************************************
*/

int  BSP_PreInit (void)
{           
	//BSP_CPU_Init();                                            /* Initialize CPU/System Clocks        */
    CSP_IntInit();                                           /* Initialize Interrupt Controller module. */
	OLED_init();
    /*
     * Configure MSS GPIOs.
     * See above mapping
     */
    CSP_GPIO_Cfg((CSP_DEV_NBR   )CSP_GPIO_PORT_NBR_00,          /* See note #1 & #2.                                    */
                 (CSP_GPIO_MSK  )BSP_GPIO_BREADBOARD_GRP ,
                 (CSP_OPT       )CSP_GPIO_DIR_OUT,
                 (CSP_OPT_FLAGS )CSP_GPIO_FLAG_MODE_NONE,
                 (CPU_BOOLEAN   )DEF_NO,
                 (CSP_OPT       )0,
                 (CSP_OPT       )CSP_GPIO_FNCT_00);

    CSP_GPIO_Cfg((CSP_DEV_NBR   )CSP_GPIO_PORT_NBR_01,          /* See note #1 & #2.                                    */
                 (CSP_GPIO_MSK  )BSP_GPIO_PB_GRP ,
                 (CSP_OPT       )CSP_GPIO_DIR_IN,
                 (CSP_OPT_FLAGS )CSP_GPIO_FLAG_MODE_NONE,
                 (CPU_BOOLEAN   )DEF_NO,
                 (CSP_OPT       )0,
                 (CSP_OPT       )CSP_GPIO_FNCT_00);

#if MODBUS_ACTIVE
#else
    MSS_UART_init( &g_mss_uart0,
                   MSS_UART_57600_BAUD,
                   MSS_UART_DATA_8_BITS | MSS_UART_NO_PARITY | MSS_UART_ONE_STOP_BIT );
#endif


    UART_init( &g_uart2,
    		   UART2_BASE,
               BAUD_VALUE_57600,
               (DATA_8_BITS | NO_PARITY) );

    BSP_INIT_CNTR();			/* Initialize 32-bit counter */

    BSP_LED_Off(0);                                             /* Turn Off LEDs */

	OLED_set_cursor( FIRST_LINE, FIRST_CHARACTER );
	OLED_write_string("      SenSEI       ");

	return 1;
}



/*
*********************************************************************************************************
*                                             BSP_PostInit()
*
* Description : Initialize peripherals & modules that requires OS services.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  BSP_PostInit (void)
{        
    CPU_TS_TmrInit();
}


/*
*********************************************************************************************************
*                                             BSP_CPU_Init()
*
* Description : Initialize the CPU clks, memory map, etc.
*                   (1) Remove the reset condition for GPIO.
*                   (2) Perform the REMAP based on the configuration.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : BSP_PreInit()
*
* Note(s)     : (1) The CLK initialization is done trough the Actel's Libero Integated Design Enviroment.
*********************************************************************************************************
*/

static  CPU_BOOLEAN  BSP_CPU_Init (void)
{
    CPU_INT32U  reg_val;

#ifndef APP_CFG_CODE_IRAM_EN
    CPU_INT32U  cpu_freq;
#endif    
    
    
    BSP_REG_WDOGENABLE = BSP_VAL_WDOGENABLE_DIS;                /* Disable watchdog timer                               */
    
    reg_val            = BSP_REG_SOFT_RST_CR;
    
    DEF_BIT_CLR(reg_val, BSP_BIT_SOFT_RST_CR_GPIO_SR);
        
    BSP_REG_SOFT_RST_CR = reg_val;
    
#ifdef APP_CFG_CODE_IRAM_EN                                     /* If code is exceuted  on IRAM.                        */
    DEF_BIT_SET(BSP_REG_ESRAM_CR, BSP_BIT_ESRAM_CR_COM_REMAP);  /* ... Remmap eSRAM_0 to address 0x00000000.            */
#else    
    DEF_BIT_CLR(BSP_REG_ESRAM_CR, BSP_BIT_ESRAM_CR_COM_REMAP);
    
    cpu_freq = CSP_PM_CPU_ClkFreqGet();
    reg_val  = BSP_REG_ENVM_CR;
    
    if (cpu_freq >= 80000000) {
        DEF_BIT_CLR(reg_val, (BSP_BIT_ENVM_CR_SIX_CYCLE | BSP_BIT_ENVM_CR_PIPE_BYPASS));    
    } else {
        DEF_BIT_CLR(reg_val, BSP_BIT_ENVM_CR_SIX_CYCLE);
        DEF_BIT_SET(reg_val, BSP_BIT_ENVM_CR_PIPE_BYPASS);
    }
    
    DEF_BIT_CLR(reg_val, BSP_MSK_ENVM_CR_REMAP_SIZE);
    DEF_BIT_SET(reg_val, BSP_BIT_ENVM_CR_REMAP_SIZE_64);
    BSP_REG_ENVM_CR = reg_val;
    
    reg_val         =  BSP_REG_ENVM_REMAP_SYS_CR;
    DEF_BIT_CLR(reg_val, BSP_MSK_ENVM_REMAP_SYS_CR_REMAP_BASE);
    DEF_BIT_SET(reg_val, BSP_BIT_ENVM_REMAP_SYS_CR_REMAP_EN);        
    
    BSP_REG_ENVM_REMAP_SYS_CR = reg_val;
#endif
    
    return (DEF_OK);
}


/*
*********************************************************************************************************
*                                                 BSP_LED_Off()
*
* Description : Turn OFF any or all the LEDs on the board.
*
* Argument(s) : led    The ID of the LED to control:
*
*                           0    turn OFF all LEDs on the board.
*                           1    turn OFF LED1.
*                           2    turn OFF LED2.
*                           3    turn OFF LED3.
*                           4    turn OFF LED4.
*                           5    turn OFF LED5.
*                           6    turn OFF LED6.
*                           7    turn OFF LED7.
*                           8    turn OFF LED8.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  BSP_LED_Off (CPU_INT08U  led) 
{
    if (led == 0u) {
        CSP_GPIO_BitSet(CSP_GPIO_PORT_NBR_00,
                        BSP_GPIO_LED_GRP);
        
    } else if (led <= 8u) {
        CSP_GPIO_BitSet(CSP_GPIO_PORT_NBR_00,
                        DEF_BIT(led - 1u));
    } else {
        ;
    }
}


/*
*********************************************************************************************************
*                                                 BSP_LED_On()
*
* Description : Turn ON any or all the LEDs on the board.
*
* Argument(s) : led     The ID of the LED to control:
*                           0    turn ON all LEDs on the board.
*                           1    turn ON LED1.
*                           2    turn ON LED2.
*                           3    turn ON LED3.
*                           4    turn ON LED4.
*                           5    turn ON LED5.
*                           6    turn ON LED6.
*                           7    turn ON LED7.
*                           8    turn ON LED8.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  BSP_LED_On (CPU_INT08U  led) 
{
    if (led == 0u) {
        CSP_GPIO_BitClr(CSP_GPIO_PORT_NBR_00,
                        BSP_GPIO_LED_GRP);
        
    } else if (led <= 8u) {
        CSP_GPIO_BitClr(CSP_GPIO_PORT_NBR_00,
                        DEF_BIT(led - 1u));
    } else {
        ;
    }
}


/*
*********************************************************************************************************
*                                                 BSP_LED_Toggle()
*
* Description : Toggles any or all the LEDs on the board.
*
* Argument(s) : led_id   The ID of the LED to control:
*
*                       0    toggle all LEDs on the board
*                       1    toggle LED1
*                       2    toggle LED2
*                       3    toggle LED3
*                       4    toggle LED4
*                       5    toggle LED5
*                       6    toggle LED6
*                       7    toggle LED7
*                       8    toggle LED8
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  BSP_LED_Toggle (CPU_INT08U  led) 
{
    if (led == 0u) {
        CSP_GPIO_BitToggle(CSP_GPIO_PORT_NBR_00,
                           BSP_GPIO_LED_GRP);
        
    } else if (led <= 8u) {
        CSP_GPIO_BitToggle(CSP_GPIO_PORT_NBR_00,
                           DEF_BIT(led - 1u));
    } else {
        ;
    }

}


/*
*********************************************************************************************************
*                                            BSP_PB_GetStatus()
*
* Description : Get the status of the board's push buttons.
*
* Argument(s) : pb   Push button Identifier
*
*                       1  Push button 1 (SW1)
*                       2  Push button 2 (SW2)

*
* Return(s)   : DEF_ON   If the push button is pressed
*               DEF_OFF, otherwise.
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

CPU_BOOLEAN  BSP_PB_GetStatus (CPU_INT08U pb)
{
    CSP_GPIO_MSK   port_val;
    CPU_BOOLEAN    status;
    
    
    status   = DEF_OFF;    
    port_val = CSP_GPIO_Rd(CSP_GPIO_PORT_NBR_00);
        
    switch (pb) {
        case 1u:
             if (DEF_BIT_IS_CLR(port_val, BSP_GPIO_PB_00)) {
                 status = DEF_ON;
         	     CSP_GPIO_BitSet(0,DEF_BIT_12);
//         	     MSS_UART_polled_tx( &g_mss_uart0, message_SovDiagEnabled, sizeof(message_SovDiagEnabled) ); /* Send message through UART0	     */
         		 OSTimeDlyHMSM(0,0,3,0);
         		 CSP_GPIO_BitClr(0,DEF_BIT_12);
             }
             break;
                 
        case 2u:
             if (DEF_BIT_IS_CLR(port_val, BSP_GPIO_PB_01)){
                 status = DEF_ON;
                 CSP_GPIO_BitSet(0,DEF_BIT_13);
//         	     MSS_UART_polled_tx( &g_mss_uart0, message_SovNormEnabled, sizeof(message_SovNormEnabled) ); /* Send message through UART0	     */
         		 OSTimeDlyHMSM(0,0,3,0);
         		 CSP_GPIO_BitClr(0,DEF_BIT_13);
             }
             break;
             
        default:
             break;        
    }
 
    return (status);
}

void BSP_INIT_CNTR()
{
    /* Clear Fabric Counter */
	HW_set_32bit_reg(CNTR0_BASE, 0x00000003);
	HW_set_32bit_reg(CNTR0_BASE, 0x00000002);

	HW_set_32bit_reg(CNTR0_BASE+0x08, 0x00000123);
	HW_set_32bit_reg(CNTR0_BASE, 0x00000000);
}

