/*
*********************************************************************************************************
*                                     MICRIUM BOARD SUPPORT SUPPORT
*
*                          (c) Copyright 2003-2009; Micrium, Inc.; Weston, FL
*
*               All rights reserved.  Protected by international copyright laws.
*               Knowledge of the source code may NOT be used to develop a similar product.
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                      BOARD SUPPORT PACKAGE (BSP)
*
*                                      SMARTFUSION EVALUATION KIT
*
* Filename      : bsp.h
* Version       : V1.00
* Programmer(s) : FT
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                                 MODULE
*
* Note(s) : (1) This header file is protected from multiple pre-processor inclusion through use of the
*               BSP present pre-processor macro definition.
*********************************************************************************************************
*/

#ifndef  BSP_PRESENT
#define  BSP_PRESENT


/*
*********************************************************************************************************
*                                            INCLUDE FILES
*********************************************************************************************************
*/

#include  <cpu.h>
#include  <cpu_core.h>

#include  <lib_def.h>
#include  <lib_ascii.h>

#include  <csp.h>

#include  <includes.h>

/*
*********************************************************************************************************
*                                             REGISTER & BIT DEFINES
*********************************************************************************************************
*/

#define  BSP_REG_WDOGENABLE                     (* (CPU_REG32 *)(0x40006010))

#define  BSP_REG_SOFT_RST_CR                    (* (CPU_REG32 *)(0xE0042030))
#define  BSP_REG_ESRAM_CR                       (* (CPU_REG32 *)(0xE0042000))
#define  BSP_REG_ENVM_CR                        (* (CPU_REG32 *)(0xE0042004))
#define  BSP_REG_ENVM_REMAP_SYS_CR              (* (CPU_REG32 *)(0xE0042008))


#define  BSP_BIT_SOFT_RST_CR_GPIO_SR            DEF_BIT_14

    
#define  BSP_BIT_ENVM_CR_SIX_CYCLE              DEF_BIT_07  
#define  BSP_BIT_ENVM_CR_PIPE_BYPASS            DEF_BIT_06
#define  BSP_MSK_ENVM_CR_REMAP_SIZE             DEF_BIT_FIELD(5u, 0u)
#define  BSP_BIT_ENVM_CR_REMAP_SIZE_64         (DEF_BIT_04 | DEF_BIT_00)
#define  BSP_MSK_ENVM_REMAP_SYS_CR_REMAP_BASE   DEF_BIT_FIELD(19u, 1u)
#define  BSP_BIT_ENVM_REMAP_SYS_CR_REMAP_EN     DEF_BIT_00   

#define  BSP_VAL_WDOGENABLE_DIS                 0x4C6E55FA          


/*
*********************************************************************************************************
*                                             GPIO DEFINES
*********************************************************************************************************
*/

#define  BSP_GPIO_LED_00                        DEF_BIT_00
#define  BSP_GPIO_LED_01                        DEF_BIT_01
#define  BSP_GPIO_LED_02                        DEF_BIT_02
#define  BSP_GPIO_LED_03                        DEF_BIT_03
#define  BSP_GPIO_LED_04                        DEF_BIT_04
#define  BSP_GPIO_LED_05                        DEF_BIT_05
#define  BSP_GPIO_LED_06                        DEF_BIT_06
#define  BSP_GPIO_LED_07                        DEF_BIT_07


#define  BSP_GPIO_LED_GRP                      (BSP_GPIO_LED_00 | \
                                                BSP_GPIO_LED_01 | \
                                                BSP_GPIO_LED_02 | \
                                                BSP_GPIO_LED_03 | \
                                                BSP_GPIO_LED_04 | \
                                                BSP_GPIO_LED_05 | \
                                                BSP_GPIO_LED_06 | \
                                                BSP_GPIO_LED_07)

#define  BSP_GPIO_LED_BREADBOARD                BSP_GPIO_LED_00

#define  BSP_GPIO_PB_00                         DEF_BIT_14
#define  BSP_GPIO_PB_01                         DEF_BIT_15
#define  BSP_GPIO_PB_GRP                       (BSP_GPIO_PB_00 | \
                                                BSP_GPIO_PB_01)

#define  BSP_BIT_ESRAM_CR_COM_REMAP             DEF_BIT_00

#define  BSP_GPIO_SOV_BREADBOARD               (DEF_BIT_12 | \
												DEF_BIT_13)

#define BSP_GPIO_BREADBOARD_GRP				   (BSP_GPIO_SOV_BREADBOARD |\
												BSP_GPIO_LED_BREADBOARD)

/*
*********************************************************************************************************
*                                                 EXTERNS
*********************************************************************************************************
*/


#ifdef   BSP_MODULE
#define  BSP_EXT
#else
#define  BSP_EXT  extern
#endif


/*
*********************************************************************************************************
*                                        DEFAULT CONFIGURATION
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                      CONDITIONAL INCLUDE FILES
*********************************************************************************************************
*/

#define  BSP_BOARD_NAME                         "SMARTFUSION2"
#define	 BSP_BOARD_NUMBER						0xA2F2


/*
*********************************************************************************************************
*                                            BOARDS LEDs DEFINES
*********************************************************************************************************
*/

#define  BSP_LED_ALL                                      0u
#define  BSP_LED_NBR_01                                   1u
#define  BSP_LED_NBR_02                                   2u
#define  BSP_LED_NBR_03                                   3u
#define  BSP_LED_NBR_04                                   4u
#define  BSP_LED_NBR_05                                   5u
#define  BSP_LED_NBR_06                                   6u
#define  BSP_LED_NBR_07                                   7u

#define A2F200M3F_CoreUART_CTRL2_BIT8			DEF_BIT_00
#define A2F200M3F_CoreUART_CTRL2_PARITY_EN		DEF_BIT_01
#define A2F200M3F_CoreUART_CTRL2_ODD_N_EVEN		DEF_BIT_02
#define A2F200M3F_CoreUART_STATUS_TXRDY			DEF_BIT_00
#define A2F200M3F_CoreUART_STATUS_RXRDY			DEF_BIT_01
#define A2F200M3F_CoreUART_STATUS_PARITY_ERR	DEF_BIT_02
#define A2F200M3F_CoreUART_STATUS_OVERFLOW		DEF_BIT_03
#define A2F200M3F_CoreUART_STATUS_FRAMING_ERR	DEF_BIT_04

/* ------ UART MODE (US_CR) REGISTER BIT DEFINES ------- */
#define A2F200M3F_UART_MR_PAR_ENABLED           (0x01 <<  1)
#define A2F200M3F_UART_MR_PAR_DISABLED          (0x00 <<  1)

#define A2F200M3F_UART_MR_PAR_EVEN              (0x00 <<  2)
#define A2F200M3F_UART_MR_PAR_ODD               (0x01 <<  2)
#define A2F200M3F_UART_MR_PAR_NONE              (0x00 <<  0)


#define A2F200M3F_UART_MR_NBSTOP_1             (0x00 << 0)
#define A2F200M3F_UART_MR_NBSTOP_2             (0x00 << 0)

#define A2F200M3F_UART_MR_CHRL_7                (0x00 <<  0)
#define A2F200M3F_UART_MR_CHRL_8                (0x01 <<  0)

/*
*********************************************************************************************************
*                                          GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                         FUNCTION PROTOTYPES
*********************************************************************************************************
*/

int          BSP_PreInit            (void);
void         BSP_PostInit           (void);
void         BSP_LowLevelInit       (void);


/*
*********************************************************************************************************
*                                             LED SERVICES
*********************************************************************************************************
*/

void         BSP_LED_On             (CPU_INT08U   led);
void         BSP_LED_Off            (CPU_INT08U   led);
void         BSP_LED_Toggle         (CPU_INT08U   led);


/*
*********************************************************************************************************
*                                         PUSH BUTTON SERVICES
*********************************************************************************************************
*/

CPU_BOOLEAN  BSP_PB_GetStatus       (CPU_INT08U   pb);


/*
*********************************************************************************************************
*                                          CONFIGURATION ERRORS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                              MODULE END
*********************************************************************************************************
*/

#endif    
