/*
*********************************************************************************************************
*                                              uC/Modbus
*                                       The Embedded Modbus Stack
*
*                          (c) Copyright 2003-2008; Micrium, Inc.; Weston, FL               
*
*               All rights reserved.  Protected by international copyright laws.
*               Knowledge of the source code may NOT be used to develop a similar product.
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                            uC/Modbus
*
*                                      Board Support Package
*                                     Atmel AT91SAM9XE (ARM9)
*
* Filename    : mb_bsp.c
* Version     : V2.12
* Programmers : FT
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                            INCLUDE FILES
*********************************************************************************************************
*/

#include    <includes.h>

/*
*********************************************************************************************************
*                                            LOCAL DEFINES
*********************************************************************************************************
*/
/* ------------------ MSS BASE REGISTER ADDRESS ---------------- */
#define  A2F200M3F_UART0_BASE_ADDR     ((CPU_INT32U)0x40000000)
#define  A2F200M3F_UART1_BASE_ADDR     ((CPU_INT32U)0x40010000)
/* -------------- FPGA Fabric BASE REGISTER ADDRESS ------------ */
#define  A2F200M3F_UART2_BASE_ADDR     ((CPU_INT32U)0x40050100)

/* -------------- FPGA Fabric BASE REGISTER ADDRESS ------------ */
#define  A2F200M3F_TC0_BASE_ADDR       ((CPU_INT32U)0x40005000)
#define  A2F200M3F_TC1_BASE_ADDR       ((CPU_INT32U)0x40005018)
#define  A2F200M3F_TC3_BASE_ADDR       ((CPU_INT32U)0x40005030)

/* -------------------- Peripheral IDs -------------------- */
#define  A2F200M3F_PER_ID_SYS                      1

#define  A2F200M3F_PER_ID_UART0                    6
#define  A2F200M3F_PER_ID_UART1                    7
#define  A2F200M3F_PER_ID_UART2                    8

#define  A2F200M3F_PER_ID_TC0                      17
#define  A2F200M3F_PER_ID_TC1                      18


/*
*********************************************************************************************************
*                                            A2F200M3F REGISTER BIT DEFINES
*********************************************************************************************************
*/



#define BAUDVALUE_LSB ( (uint16_t) (0x00FF) )
#define BAUDVALUE_MSB ( (uint16_t) (0xFF00) )
#define BAUDVALUE_SHIFT ( (uint8_t) (5) )

/*
*********************************************************************************************************
*                                        REGISTER DEFINITIONS
*
* Note(s) : (1) The device register definition structure MUST take into account appropriate
*               register offsets and apply reserved space as required.  The registers listed
*               within the register definition structure MUST reflect the exact ordering and
*               data sizes illustrated in the device user guide.
*********************************************************************************************************
*/
                                                                /* +------------|---------------------|--------------------+ */
                                                                /* |Offset      | Register            | Name               | */
                                                                /* |------------|---------------------|--------------------| */
                                                                /* |========= UASART/DBGU UNIT REGISTERS ================  | */
typedef  struct  a2f200m3f_uart_mss {                           /* +------------|---------------------|--------------------+ */
    CPU_INT08U   US_RBR;                                        /* |0x0000      | Buffer Reg          |RBR                 | */
    CPU_INT08U   US_THR;                                        /* |0x0000      | Transmit Holding Reg|THR                 | */
    CPU_INT08U   US_DLR;                                        /* |0x0000      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   RESERVED_UART01;                               /* |0x0000      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   US_DMR;                                        /* |0x0004      | Divisor Latch (LSB) |DMR                 | */
    CPU_INT08U   US_IER;                                        /* |0x0004      | Interrupt Enable Reg|IER                 | */
    CPU_INT08U   RESERVED_UART02;                               /* |0x0004      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   RESERVED_UART03;                               /* |0x0004      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   US_IIR;                                        /* |0x0008      | Interrupt ID Reg    |IIR                 | */
    CPU_INT08U   US_FCR;                                        /* |0x0008      | FIFO Control Reg    |FCR                 | */
    CPU_INT08U   RESERVED_UART04;                               /* |0x0008      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   RESERVED_UART05;                               /* |0x0008      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   US_LCR;                                        /* |0x000C      | Line Control Reg    |LCR                 | */
    CPU_INT08U   RESERVED_UART06;                               /* |0x000C      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   RESERVED_UART07;                               /* |0x000C      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   RESERVED_UART08;                               /* |0x000C      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   US_MCR;                                        /* |0x0010      | Modem Control Reg   |MCR                 | */
    CPU_INT08U   RESERVED_UART10;                               /* |0x0010      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   RESERVED_UART11;                               /* |0x0010      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   RESERVED_UART12;                               /* |0x0010      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   US_LSR;                                        /* |0x0014      | Line Status Reg     |LSR                 | */
    CPU_INT08U   RESERVED_UART13;                               /* |0x0014      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   RESERVED_UART14;                               /* |0x0014      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   RESERVED_UART15;                               /* |0x0014      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   US_MSR;                                        /* |0x0018      | Modem Status Reg    |MSR                 | */
    CPU_INT08U   RESERVED_UART16;                               /* |0x0018      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   RESERVED_UART17;                               /* |0x0018      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   RESERVED_UART18;                               /* |0x0018      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   US_SR;                                         /* |0x001C      | Scratch Reg         |SR                  | */
    CPU_INT08U   RESERVED_UART19;                               /* |0x001C      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   RESERVED_UART20;                               /* |0x001C      | Divisor Latch (LSB) |DLR                 | */
    CPU_INT08U   RESERVED_UART21;                               /* |0x001C      | Divisor Latch (LSB) |DLR                 | */
} A2F200M3F_UART_MSS;                                           /* +------------|---------------------|--------------------+ */

typedef struct a2f200m3f_tc    {                                /* +------------|-------------------------------------------+ */
    CPU_INT32U  TIM1_VAL;                                       /* |0x0000      | Current value of Timer 1                  | */
    CPU_INT32U  TIM1_LOADVAL;                                   /* |0x0004      | Load value for Timer 1                    | */
    CPU_INT32U  TIM1_BGLOADVAL;                                 /* |0x0008      | Background load value for Timer 1         | */
    CPU_INT32U  TIM1_CTRL;                                      /* |0x000C      | Timer 1 Control Register                  | */
    CPU_INT32U  TIM1_RIS;                                       /* |0x0010      | Timer 1 raw interrupt status              | */
    CPU_INT32U  TIM1_MIS;                                       /* |0x0014      | Timer 1 masked interrupt status           | */
    CPU_INT32U  TIM2_VAL;                                       /* |0x0018      | Current value of Timer 2                  | */
    CPU_INT32U  TIM2_LOADVAL;                                   /* |0x001C      | Load value for Timer 2                    | */
    CPU_INT32U  TIM2_BGLOADVAL;                                 /* |0x0020      | Background load value for Timer 2         | */
    CPU_INT32U  TIM2_CTRL;                                      /* |0x0024      | Timer 2 Control Register                  | */
    CPU_INT32U  TIM2_RIS;                                       /* |0x0028      | Timer 2 raw interrupt status              | */
    CPU_INT32U  TIM2_MIS;                                       /* |0x002C      | Timer 2 masked interrupt status           | */
    CPU_INT32U  TIM64_VAL_U;                                    /* |0x0030      | Upper 32-bit word in 64-bit mode          | */
    CPU_INT32U  TIM64_VAL_L;                                    /* |0x0034      | Lower 32-bit word in 64-bit mode          | */
    CPU_INT32U  TIM64_LOADVAL_U;                                /* |0x0038      | Upper 32-bit load value word in 64-bit    | */
    CPU_INT32U  TITM64_LOADVAL_L;                               /* |0x003C      | Lower 32-bit load value word in 64-bit    | */
    CPU_INT32U  TIM64_BGLOADVAL_U;                              /* |0x0040      | Upper 32-bit background load value in 64-bit mode         | */
    CPU_INT32U  TIM64_BGLOADVAL_L;                              /* |0x0044      | Lower 32-bit background load value in 64-bit mod        | */
    CPU_INT32U  TIM64_CTRL;                                     /* |0x0048      | Control Register in 64-bit mode              | */
    CPU_INT32U  TIM64_RIS;                                      /* |0x004C      | Raw interrupt status in 64-bit mode                 | */
    CPU_INT32U  TIM64_MIS;                                      /* |0x0050      | Masked interrupt status in 64-bit mode   | */
    CPU_INT32U  TIM64_MODE;                                     /* |0x0054      | System Timer dual 32-bit or 64-bit mode          | */
} A2F200M3F_TC;                                                 /* +------------|-------------------------------------|---------------+ */


/* Interrupt Enable Register (IER) */
#define MSS_UART_IER_ERBFI		1
#define MSS_UART_IER_ETBEI		2
#define MSS_UART_IER_ELSI		4
#define MSS_UART_IER_EDSSI		8

#define MSS_UART_FCR_CLEAR_RX_FIFO			2
#define MSS_UART_FCR_CLEAR_TX_FIFO			4
#define MSS_UART_FCR_ENABLE_TXRDY_RXRDY		8
#define MSS_UART_DATA_READY 				( (uint8_t) 0x01 )

																/* +------------|---------------------|--------------------+ */
                                                                /* |Offset      | Register            | Name               | */
                                                                /* |------------|---------------------|--------------------| */
                                                                /* |========= UASART/DBGU UNIT REGISTERS ================  | */
typedef  struct  a2f200m3f_uart     {                           /* +------------|---------------------|--------------------+ */
    CPU_INT32U  US_TXDATA;                                      /* |0x0000      | Transmit Data Reg   |TXDATA              | */
    CPU_INT32U  US_RXDATA;                                      /* |0x0004      | Receive Data Reg    |RXDATA              | */
    CPU_INT32U  US_CTRL1;                                       /* |0x0008      | Control Register 1  |CTRL1               | */
    CPU_INT32U  US_CTRL2;                                       /* |0x000C      | Control Register 2  |CTRL2               | */                                         
    CPU_INT32U  US_STATUS;                                      /* |0x0010      | Status              |STATUS              | */
} A2F200M3F_UART;                                               /* +------------|---------------------|--------------------+ */
                                                                /* |================= AIC REGISTERS =====================  | */

/*
*********************************************************************************************************
*                                    LOCAL CONFIGURATION ERRORS
*********************************************************************************************************
*/
                                                                /*-------------- UART MODE CONFIGURATION ERRORS ------------ */
#ifndef MB_BSP_CFG_UART_00_MODE
    #error  "MB_BSP_CFG_UART_00_MODE           not #define'd in 'app_cfg.h'          "
    #error  "                                  [MUST be  MB_BSP_UART_RS232_MODE   ]  "
    #error  "                                  [     ||  MB_BSP_UART_RS485_MODE   ]  "

#elif   (MB_BSP_CFG_UART_00_MODE != MB_BSP_UART_RS232_MODE  ) && \
        (MB_BSP_CFG_UART_00_MODE != MB_BSP_UART_RS485_MODE  ) 
    #error  "MB_BSP_CFG_UART_00_MODE           illegally #define'd in 'app_cfg.h'    "
    #error  "                                  [MUST be  MB_BSP_UART_RS232_MODE   ]  "
    #error  "                                  [     ||  MB_BSP_UART_RS485_MODE   ]  "
#endif

#ifndef MB_BSP_CFG_UART_01_MODE
    #error  "MB_BSP_CFG_UART_01_MODE           not #define'd in 'app_cfg.h'          "
    #error  "                                  [MUST be  MB_BSP_UART_RS232_MODE   ]  "
    #error  "                                  [     ||  MB_BSP_UART_RS485_MODE   ]  "

#elif   (MB_BSP_CFG_UART_01_MODE != MB_BSP_UART_RS232_MODE  ) && \
        (MB_BSP_CFG_UART_01_MODE != MB_BSP_UART_RS485_MODE  ) 
    #error  "MB_BSP_CFG_UART_01_MODE           illegally #define'd in 'app_cfg.h'    "
    #error  "                                  [MUST be  MB_BSP_UART_RS232_MODE   ]  "
    #error  "                                  [     ||  MB_BSP_UART_RS485_MODE   ]  "
#endif

#ifndef MB_BSP_CFG_UART_02_MODE
    #error  "MB_BSP_CFG_UART_01_MODE           not #define'd in 'app_cfg.h'          "
    #error  "                                  [MUST be  MB_BSP_UART_RS232_MODE   ]  "
    #error  "                                  [     ||  MB_BSP_UART_RS485_MODE   ]  "

#elif   (MB_BSP_CFG_UART_02_MODE != MB_BSP_UART_RS232_MODE  ) && \
        (MB_BSP_CFG_UART_02_MODE != MB_BSP_UART_RS485_MODE  ) 
    #error  "MB_BSP_CFG_UART_01_MODE           illegally #define'd in 'app_cfg.h'    "
    #error  "                                  [MUST be  MB_BSP_UART_RS232_MODE   ]  "
    #error  "                                  [     ||  MB_BSP_UART_RS485_MODE   ]  "
#endif

                                                                /*-------------- TIMER SEL CONFIGURATION  ------------ */
#ifndef MS_BSP_CFG_TMR
    #error  "MS_BSP_CFG_TMR           not #define'd in 'app_cfg.h'                  "
    #error  "                                  [MUST be  MB_BSP_TMR_0   ]           "
    #error  "                                  [     ||  MB_BSP_TMR_1   ]           "
    #error  "                                  [     ||  MB_BSP_TMR_2   ]           "

#elif   (MS_BSP_CFG_TMR != MB_BSP_TMR_0 ) && \
        (MS_BSP_CFG_TMR != MB_BSP_TMR_1 ) && \
        (MS_BSP_CFG_TMR != MB_BSP_TMR_2 )
    #error  "MS_BSP_CFG_TMR           illegally #define'd in 'app_cfg.h'          "
    #error  "                                  [MUST be  MB_BSP_TMR_0   ]         "
    #error  "                                  [     ||  MB_BSP_TMR_1   ]         "
    #error  "                                  [     ||  MB_BSP_TMR_2   ]         "

#endif


/*
*********************************************************************************************************
*                                             LOCAL VARIABLES
*********************************************************************************************************
*/

#if (MODBUS_CFG_RTU_EN == DEF_ENABLED)
static  CPU_INT32U  MB_Tmr_ReloadCnts;
#endif

#define TX_COMPLETE         0U
#define NULL_HANDLER        ( (mss_uart_irq_handler_t) 0)
#define TX_FIFO_SIZE        16U

mss_uart_instance_t			p_mb_usart0;
mss_uart_instance_t			p_mb_usart1;

extern  MODBUS_CH   	*App_ModbusCH1;


void  MB_CommRxTxISR_Handler (CPU_INT08U  port_nbr);
/*$PAGE*/
/*
*********************************************************************************************************
*                                             MB_CommExit()
*
* Description : This function is called to terminate Modbus communications.  All Modbus channels are close.
*
* Argument(s) : none
*
* Return(s)   : none.
*
* Caller(s)   : MB_Exit()
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  MB_CommExit (void)
{
    CPU_INT08U   ch;
    MODBUS_CH   *pch;


    pch = &MB_ChTbl[0];
    for (ch = 0; ch < MODBUS_CFG_MAX_CH; ch++) {
        MB_CommTxIntDis(pch);
        MB_CommRxIntDis(pch);
        pch++;
    }
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                           MB_CommPortCfg()
*
* Description : This function initializes the serial port to the desired baud rate and the UART will be
*               configured for N, 8, 1 (No parity, 8 bits, 1 stop).
*
* Argument(s) : pch        is a pointer to the Modbus channel
*               port_nbr   is the desired serial port number.  This argument allows you to assign a
*                          specific serial port to a sepcific Modbus channel.
*               baud       is the desired baud rate for the serial port.
*               parity     is the desired parity and can be either:
*
*                          MODBUS_PARITY_NONE
*                          MODBUS_PARITY_ODD
*                          MODBUS_PARITY_EVEN
*
*               bits       specifies the number of bit and can be either 7 or 8.
*               stops      specifies the number of stop bits and can either be 1 or 2
*
* Return(s)   : none.
*
* Caller(s)   : MB_CfgCh()
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  MB_CommPortCfg (MODBUS_CH  *pch,
                      CPU_INT08U  port_nbr,
                      CPU_INT32U  baud,
                      CPU_INT08U  bits,
                      CPU_INT08U  parity,
                      CPU_INT08U  stops)
{
    CPU_INT32U         	dev_usart_mr;
    CPU_INT08U         	dev_per_id;

    uint32_t 			baud_value;
    uint32_t 			pclk_freq;

    mss_uart_instance_t*	p_mb_usart;

    baud_value = 0;
    pclk_freq  = 0;

    /* Force the value of the CMSIS global variables holding the various system
     * clock frequencies to be updated. */
    SystemCoreClockUpdate();

    if (pch != (MODBUS_CH *)0)
    {
        pch->PortNbr        = port_nbr;                             /* Store configuration in channel              */
        pch->BaudRate       = baud;
        pch->Parity         = parity;
        pch->Bits           = bits;
        pch->Stops          = stops;
                                                                    /* ------ USART CONFIGURATION SETUP ----------  */
        switch (parity)
        {                                             				/* (1) Setup USART parity                       */
            case MODBUS_PARITY_ODD:
                 dev_usart_mr = A2F200M3F_UART_MR_PAR_ODD;
                 break;

            case MODBUS_PARITY_EVEN:
                 dev_usart_mr = A2F200M3F_UART_MR_PAR_EVEN;
                 break;

            case MODBUS_PARITY_NONE:
            default:
                 dev_usart_mr = A2F200M3F_UART_MR_PAR_NONE;
                 break;
        }

        if (bits == 7)
        {                                              				/* (2) Setup #bits                               */
            dev_usart_mr = A2F200M3F_UART_MR_CHRL_7;
        } else
        {
            dev_usart_mr = A2F200M3F_UART_MR_CHRL_8;
        }

		// Not supported on CoreUARTapb
        if (stops == 2)
        {                                             				/* (3) Setup # stops bits                        */
            dev_usart_mr |= A2F200M3F_UART_MR_NBSTOP_2;
        }
                                                                    /* (4) Setup UASRT baud rate                     */

        /* ----------- USART PORT SELECT --------------- */
        switch (port_nbr) {
            default:
            case MB_BSP_UART_00:
                 p_mb_usart		   = &p_mb_usart0;
                 dev_per_id        = (CPU_INT08U        )(A2F200M3F_PER_ID_UART0);
                 break;

            case MB_BSP_UART_01:
                 p_mb_usart		   = &p_mb_usart1;
                 dev_per_id        = (CPU_INT08U        )(A2F200M3F_PER_ID_UART1);
                 break;

            case MB_BSP_UART_02:

            	break;
        }

        if ( port_nbr == MB_BSP_UART_00 )
        {
        	p_mb_usart->hw_reg = UART0;
        	p_mb_usart->hw_reg_bit = UART0_BITBAND;
        	p_mb_usart->irqn = UART0_IRQn;

            pclk_freq = g_FrequencyPCLK0;

            /* reset UART0 */
            SYSREG->SOFT_RST_CR |= SYSREG_UART0_SOFTRESET_MASK;
            /* Clear any previously pended UART0 interrupt */
            NVIC_ClearPendingIRQ( UART0_IRQn );
            /* Take UART0 out of reset. */
            SYSREG->SOFT_RST_CR &= ~SYSREG_UART0_SOFTRESET_MASK;
        }
        else
        {
        	p_mb_usart->hw_reg = UART1;
        	p_mb_usart->hw_reg_bit = UART1_BITBAND;
        	p_mb_usart->irqn = UART1_IRQn;

            pclk_freq = g_FrequencyPCLK1;

            /* Reset UART1 */
            SYSREG->SOFT_RST_CR |= SYSREG_UART1_SOFTRESET_MASK;
            /* Clear any previously pended UART1 interrupt */
            NVIC_ClearPendingIRQ( UART1_IRQn );
            /* Take UART1 out of reset. */
            SYSREG->SOFT_RST_CR &= ~SYSREG_UART1_SOFTRESET_MASK;
        }
        /* disable interrupts */
        p_mb_usart->hw_reg->IER = 0U;

        /*
         * Compute baud value based on requested baud rate and PCLK frequency.
         * The baud value is computed using the following equation:
         *      baud_value = PCLK_Frequency / (baud_rate * 16)
         * The baud value is rounded up or down depending on what would be the remainder
         * of the divide by 16 operation.
         */

        baud_value = (uint16_t) (pclk_freq / (baud));

        if ( baud_value & 0x00000008U )
        {
            /* remainder above 0.5 */
            baud_value = (baud_value >> 4U) + 1U;
        }
        else
        {
            /* remainder below 0.5 */
            baud_value = (baud_value >> 4U);
        }

        /* set divisor latch */
        p_mb_usart->hw_reg_bit->LCR_DLAB = (uint32_t)1;

        /* msb of baud value */
        p_mb_usart->hw_reg->DMR = (uint8_t)(baud_value >> 8);

        /* lsb of baud value */
        p_mb_usart->hw_reg->DLR = (uint8_t)baud_value;

        /* reset divisor latch */
        p_mb_usart->hw_reg_bit->LCR_DLAB = (uint32_t)0;

        /* set the line control register (bit length, stop bits, parity) */
        p_mb_usart->hw_reg->LCR = bits | parity | stops ;		// line_config;

        /* disable loopback */
        p_mb_usart->hw_reg_bit->MCR_LOOP = (uint32_t)0;

        /* clear receiver FIFO */
        p_mb_usart->hw_reg_bit->FCR_CLEAR_RX_FIFO = (uint32_t)1;

        /* clear transmitter FIFO */
        p_mb_usart->hw_reg_bit->FCR_CLEAR_TX_FIFO = (uint32_t)1;

        /* FIFO configuration */
        p_mb_usart->hw_reg->FCR = (uint8_t)MSS_UART_FIFO_SINGLE_BYTE + MSS_UART_FCR_ENABLE_TXRDY_RXRDY;

        /* enable RXRDYN and TXRDYN pins. The earlier FCR write to set the TX FIFO
         * trigger level inadvertently disabled the FCR_RXRDY_TXRDYN_EN bit. */
        //p_mb_usart->hw_reg_bit->FCR_RXRDY_TXRDYN_EN = (uint32_t)1;

        /* Instance setup */
        p_mb_usart->tx_buff_size = TX_COMPLETE;
        p_mb_usart->tx_buffer = (const uint8_t *)0;
        p_mb_usart->tx_idx = 0U;

        /* Default handlers for MSS UART interrupts */
        p_mb_usart->rx_handler       = MB_CommRxTxISR_Handler;			//MB_OS_RxSignal;
        p_mb_usart->tx_handler       = NULL_HANDLER;		// default_tx_handler;
        p_mb_usart->linests_handler  = NULL_HANDLER;
        p_mb_usart->modemsts_handler = NULL_HANDLER;

        /* Clear any previously pended interrupts */
        NVIC_ClearPendingIRQ( p_mb_usart->irqn );

        /* Enable RX interrupt only */
        p_mb_usart->hw_reg->IER = MSS_UART_IER_ERBFI;

        /* Enable UART instance interrupt in Cortex-M3 NVIC. */
        NVIC_EnableIRQ( p_mb_usart->irqn );

        /* Initialize the sticky status */
        p_mb_usart->status = 0U;
    }


}

/*$PAGE*/
/*
*********************************************************************************************************
*                                         MB_CommRxIntDis()
*
* Description : This function disables Rx interrupts.
*
* Argument(s) : pch        is a pointer to the Modbus channel
*
* Return(s)   : none.
*
* Caller(s)   : MB_CommExit()
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  MB_CommRxIntDis (MODBUS_CH  *pch)
{

}

/*$PAGE*/
/*
*********************************************************************************************************
*                                          MB_CommRxIntEn()
*
* Description : This function enables Rx interrupts.
*
* Argument(s) : pch        is a pointer to the Modbus channel
*
* Return(s)   : none.
*
* Caller(s)   : MB_TxByte()
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  MB_CommRxIntEn (MODBUS_CH  *pch)
{   


}

/*$PAGE*/
/*
*********************************************************************************************************
*                                       MB_CommRxTxISR_Handler()
*
* Description : This function is the ISR for either a received or transmitted character.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : This is a ISR
*
* Note(s)     : (1) The pseudo-code for this function should be:  
*
*               if (Rx Byte has been received) {
*                  c = get byte from serial port;
*                  Clear receive interrupt;
*                  pch->RxCtr++;                      Increment the number of bytes received
*                  MB_RxByte(pch, c);                 Pass character to Modbus to process
*              }
*
*              if (Byte has been transmitted) {
*                  pch->TxCtr++;                      Increment the number of bytes transmitted
*                  MB_TxByte(pch);                    Send next byte in response
*                  Clear transmit interrupt           Clear Transmit Interrupt flag
*              }
*********************************************************************************************************
*/

/***************************************************************************//**
 * UART0 interrupt service routine.
 * UART0_IRQHandler is included within the Cortex-M3 vector table as part of the
 * Fusion 2 CMSIS.
 */
#if defined(__GNUC__)
__attribute__((__interrupt__)) void MB_UART0_IRQHandler( void )
#else
static void MB_UART0_IRQHandler( void )
#endif
{
	MB_CommRxTxISR_Handler( MB_BSP_UART_00 );
    NVIC_ClearPendingIRQ( UART0_IRQn );

}

void  MB_CommRxTxISR_Handler (CPU_INT08U  port_nbr)
{

    /* ----------- USART PORT SELECT --------------- */
    switch (port_nbr) {
        default:
        case MB_BSP_UART_00:
            MB_CommRxTxISR_0_Handler();
            break;

        case MB_BSP_UART_01:
            MB_CommRxTxISR_1_Handler();
            break;

        case MB_BSP_UART_02:
			MB_CommRxTxISR_2_Handler();
        	break;
    }

}

/*$PAGE*/
/*
*********************************************************************************************************
*                                UART #0 Rx/Tx Communication handler for Modbus
*********************************************************************************************************
*/

void  MB_CommRxTxISR_0_Handler ( void )
{
	MODBUS_CH  				*pch;
	mss_uart_instance_t 	* p_mb_usart;
	uint8_t 				status;
	CPU_INT08U  			c;

    pch = App_ModbusCH1;
    p_mb_usart = &p_mb_usart0;

    status = p_mb_usart->hw_reg->LSR;
    p_mb_usart->status |= status;

    if(status & MSS_UART_DATA_READY)
    {
        c = p_mb_usart->hw_reg->RBR;
        status = p_mb_usart->hw_reg->LSR;
        p_mb_usart->status |= status;
    	pch->RxCtr++;						/* Increment the number of bytes received */
    	MB_RxByte(pch, c);					/* Pass character to Modbus to process */
    }
}


/*
*********************************************************************************************************
*                                UART #1 Rx/Tx Communication handler for Modbus
*********************************************************************************************************
*/

void  MB_CommRxTxISR_1_Handler (void)
{

}

/*
*********************************************************************************************************
*                                UART #2 Rx/Tx Communication handler for Modbus
*********************************************************************************************************
*/

void  MB_CommRxTxISR_2_Handler (void)
{

}

/*$PAGE*/
/*
*********************************************************************************************************
*                                             MB_CommTx1()
*
* Description : This function is called to obtain the next byte to send from the transmit buffer.  When
*               all bytes in the reply have been sent, transmit interrupts are disabled and the receiver
*               is enabled to accept the next Modbus request.
*
* Argument(s) : c     is the byte to send to the serial port
*
* Return(s)   : none.
*
* Caller(s)   : MB_TxByte()
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  MB_CommTx1 (MODBUS_CH  *pch,
                  CPU_INT08U  c)

{    
#if MODBUS_UART_FPGA
    volatile  A2F200M3F_UART*  		p_dev_usart;
#else
    A2F200M3F_UART_MSS* 			p_dev_usart;
    mss_uart_instance_t*			p_mb_usart;
    uint8_t 						status;
    uint32_t 						tx_size;
    uint32_t 						size_sent;
#endif

    
    switch (pch->PortNbr) {
        default:
        case MB_BSP_UART_00:
             p_dev_usart 		= (A2F200M3F_UART_MSS *)(A2F200M3F_UART0_BASE_ADDR);
             p_mb_usart		   	= &p_mb_usart0;
             break;

        case MB_BSP_UART_01:
             p_dev_usart 		= (A2F200M3F_UART_MSS *)(A2F200M3F_UART1_BASE_ADDR);
             p_mb_usart		   	= &p_mb_usart1;
             break;
             
        case MB_BSP_UART_02:
#if MODBUS_UART_FPGA
             p_dev_usart 		= (A2F200M3F_UART *)(A2F200M3F_UART2_BASE_ADDR);;
#endif
             break;

    }
#if MODBUS_UART_FPGA
    p_dev_usart->US_TXDATA = c;
#else
    /* Reset the Tx Buffer byte counters to pre-increment state */
    pch->TxBufByteCtr++;
    pch->TxCtr--;

    while (pch->TxBufByteCtr != ( (uint8_t *)0 ))
    {
        tx_size = sizeof(c);

		do {
			/* Read the Line Status Register and update the sticky record */
			status = p_mb_usart->hw_reg->LSR;
			p_mb_usart->status |= status;

			/* Check if TX FIFO is empty. */
			if( status & MSS_UART_THRE )
			{
				uint32_t fill_size = TX_FIFO_SIZE;

				/* Calculate the number of bytes to transmit. */
				if ( tx_size < TX_FIFO_SIZE )
				{
					fill_size = tx_size;
				}

				/* Fill the TX FIFO with the calculated the number of bytes. */
				for ( size_sent = 0U; size_sent < fill_size; ++size_sent )
				{
					/* Send next character in the buffer. */
					p_mb_usart->hw_reg->THR = c;
				}

				/* Calculate the number of untransmitted bytes remaining. */
				tx_size -= size_sent;
			}
		}while( tx_size );
        pch->TxBufByteCtr--;
        pch->TxCtr++;
        c = *pch->TxBufPtr++;
    }


#endif
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                         MB_CommTxIntDis()
*
* Description : This function disables Tx interrupts.
*
* Argument(s) : pch        is a pointer to the Modbus channel
*
* Return(s)   : none.
*
* Caller(s)   : MB_CommExit()
*               MB_TxByte()
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  MB_CommTxIntDis (MODBUS_CH  *pch)
{

}

/*$PAGE*/
/*
*********************************************************************************************************
*                                         MB_CommTxIntEn()
*
* Description : This function enables Tx interrupts.
*
* Argument(s) : pch        is a pointer to the Modbus channel
*
* Return(s)   : none.
*
* Caller(s)   : MB_Tx()
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  MB_CommTxIntEn (MODBUS_CH  *pch)
{

}

/*$PAGE*/
/*
*********************************************************************************************************
*                                           MB_RTU_TmrInit()
*
* Description : This function is called to initialize the RTU timeout timer.
*
* Argument(s) : freq          Is the frequency of the modbus RTU timer interrupt.
*
* Return(s)   : none.
*
* Caller(s)   : MB_Init().
*
* Note(s)     : none.
*********************************************************************************************************
*/

#if (MODBUS_CFG_RTU_EN == DEF_ENABLED)
void  MB_RTU_TmrInit (void)
{
              CPU_INT32U       dev_mclk_freq;
              CPU_INT08U       dev_id;
    volatile  A2F200M3F_TC    *p_dev_tc;

    dev_mclk_freq        = CSP_PM_CPU_ClkFreqGet();			//BSP_PMC_MclkFreqGet();
    MB_Tmr_ReloadCnts    = (dev_mclk_freq / 2) / MB_RTU_Freq;

#if   (MS_BSP_CFG_TMR == MB_BSP_TMR_0)
    p_dev_tc = (A2F200M3F_TC *)(A2F200M3F_TC0_BASE_ADDR);
    dev_id   = A2F200M3F_PER_ID_TC0;

#elif (MS_BSP_CFG_TMR == MB_BSP_TMR_1)
    p_dev_tc = (A2F200M3F_TC *)(A2F200M3F_TC1_BASE_ADDR);
    dev_id   = A2F200M3F_PER_ID_TC1;
#elif (MS_BSP_CFG_TMR == MB_BSP_TMR_2)
    p_dev_tc = (A2F200M3F_TC *)(A2F200M3F_TC2_BASE_ADDR);
    dev_id   = A2F200M3F_PER_ID_TC2;
#endif

	MSS_TIM1_init( MSS_TIMER_PERIODIC_MODE );
	MSS_TIM1_load_immediate( MB_Tmr_ReloadCnts );
	MSS_TIM1_start();
	MSS_TIM1_enable_irq();
}
#endif

/*$PAGE*/
/*
*********************************************************************************************************
*                                           MB_RTU_TmrExit()
*
* Description : This function is called to disable the RTU timeout timer.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : MB_Exit()
*
* Note(s)     : none.
*********************************************************************************************************
*/

#if (MODBUS_CFG_RTU_EN == DEF_ENABLED)
void  MB_RTU_TmrExit (void)
{


}
#endif

/*$PAGE*/
/*
*********************************************************************************************************
*                                       MB_RTU_TmrISR_Handler()
*
* Description : This function handles the case when the RTU timeout timer expires.
*
* Arguments   : none.
*
* Returns     : none.
*
* Caller(s)   : This is a ISR.
*
* Note(s)     : none.
*********************************************************************************************************
*/

#if (MODBUS_CFG_RTU_EN == DEF_ENABLED)
#if defined(__GNUC__)
__attribute__((__interrupt__)) void MB_RTU_TmrISR_Handler( void )
#else
void MB_RTU_TmrISR_Handler( void )
#endif
{
    MB_RTU_TmrCtr++;                                  /* Indicate that we had activities on this interrupt. */
    MB_RTU_TmrUpdate();                               /* Check for RTU timers that have expired             */
    MSS_TIM1_clear_irq();

}

#endif


