/*
*********************************************************************************************************
*                                                uC/CSP
*                                        CHIP SUPPORT PACKAGE
*
*                          (c) Copyright 2004-2010; Micrium, Inc.; Weston, FL               
*
*               All rights reserved.  Protected by international copyright laws.
*
*               uC/CSP is provided in source form to registered licensees ONLY.  It is 
*               illegal to distribute this source code to any third party unless you receive 
*               written permission by an authorized Micrium representative.  Knowledge of 
*               the source code may NOT be used to develop a similar product.
*
*               Please help us continue to provide the Embedded community with the finest 
*               software available.  Your honesty is greatly appreciated.
*
*               You can contact us at www.micrium.com.
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*
*                                    GENERAL PURPOSE I/O CONTROLLER
*                                       SMARTFUSION ACTEL A2Fx00
*
* Filename      : csp_gio.c
* Version       : V1.00
* Programmer(s) : FT
*********************************************************************************************************
* Note(s)       : 
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            INCLUDE FILES
*********************************************************************************************************
*/

#define    CSP_GPIO_MODULE
#include   <csp.h>

/*
*********************************************************************************************************
*                                            LOCAL DEFINES
*********************************************************************************************************
*/

                                                                /* ----------------- GPIO BASE ADDRESS ---------------- */
                                                                /* GPIO Base address                                    */  
#define  CSP_GPIO_ADDR                           (  CPU_INT32U )(0x40013000)
#define  CSP_GPIO_ADDR_IO_BANK
#define  CSP_GPIO_ADDR_GPINSRC

                                                                /* GPIO configuration register                          */
#define  CSP_GPIO_REG_GPIOCFGn(pin_nbr)          (*(CPU_REG32 *)(CSP_GPIO_ADDR + ((pin_nbr) * 4u) + 0x00))
#define  CSP_GPIO_REG_GPIOINTR                   (*(CPU_REG32 *)(CSP_GPIO_ADDR +                   + 0x80))
#define  CSP_GPIO_REG_GPI                        (*(CPU_REG32 *)(CSP_GPIO_ADDR +                   + 0x84))
#define  CSP_GPIO_REG_GPO                        (*(CPU_REG32 *)(CSP_GPIO_ADDR +                   + 0x88))

#define  CSP_GPIO_REG_IO_BANK_CFG                (*(CPU_REG32 *)(0xE0042078  ))
#define  CSP_GPIO_REG_GPIN_SRCSEL_CFG            (*(CPU_REG32 *)(0xE004207C  ))
#define  CSP_GPIO_REG_IO_MUX_CFGn(mux_nbr)       (*(CPU_REG32 *)(0xE0042100 + ((mux_nbr) * 4u)))

#define  CSP_GPIO_BIT_GPIOCFGn_GPOUTEN           DEF_BIT_00     /* Output register enabled/disabled                     */
#define  CSP_GPIO_BIT_GPIOCFGn_GPINEN            DEF_BIT_01     /* Input register enabled/disabled                      */ 
#define  CSP_GPIO_BIT_GPIOCFGn_GPO_OUTBUFEN      DEF_BIT_02     /* Output buffer enabled/disabled                       */
#define  CSP_GPIO_BIT_GPIOCFGn_GPINTEN           DEF_BIT_03     /* Interupt enabled                                     */

#define  CSP_GPIO_BIT_GPIOCFGn_GPIOINT_LOW       DEF_BIT_05     /* Interrupt type level low                             */
#define  CSP_GPIO_BIT_GPIOCFGn_GPIOINT_HIGH      DEF_BIT_NONE   /* Interrupt type level high                            */
#define  CSP_GPIO_BIT_GPIOCFGn_GPIOINT_POS       DEF_BIT_06     /* Interrupt type positive edge                         */
                                                                /* Interrupt type negative edge                         */
#define  CSP_GPIO_BIT_GPIOCFGn_GPIOINT_NEG       DEF_BIT_MASK(2u, 5u)
#define  CSP_GPIO_BIT_GPIOCFGn_GPIOINT_BOTH      DEF_BIT_07     /* Interrupt type both edges                            */

#define  CSP_GPIO_BIT_IO_MUX_CFGn_ST             DEF_BIT_09     /* Schmitt trigger enable                               */
#define  CSP_GPIO_BIT_IO_MUX_CFGn_PD             DEF_BIT_08     /* Pull down enable                                     */
#define  CSP_GPIO_BIT_IO_MUX_CFGn_PU             DEF_BIT_07     /* Pull-Up enabled                                      */

#define  CSP_GPIO_BIT_IO_MUX_CFGn_M3_S           DEF_BIT_05
#define  CSP_GPIO_BIT_IO_MUX_CFGn_M2_S_A         DEF_BIT_NONE
#define  CSP_GPIO_BIT_IO_MUX_CFGn_M2_S_0         DEF_BIT_03
#define  CSP_GPIO_BIT_IO_MUX_CFGn_M2_S_B         DEF_BIT_04
#define  CSP_GPIO_BIT_IO_MUX_CFGn_M2_S_1        (DEF_BIT_04 | DEF_BIT_03)
#define  CSP_GPIO_BIT_IO_MUX_CFGn_M1_S_A         DEF_BIT_NONE
#define  CSP_GPIO_BIT_IO_MUX_CFGn_M1_S_0         DEF_BIT_01
#define  CSP_GPIO_BIT_IO_MUX_CFGn_M1_S_B         DEF_BIT_02
#define  CSP_GPIO_BIT_IO_MUX_CFGn_M1_S_1        (DEF_BIT_02 | DEF_BIT_01)
#define  CSP_GPIO_BIT_IO_MUX_CFGn_M0_S           DEF_BIT_00

#define  CSP_GPIO_VAL_SRC_A                               0u 
#define  CSP_GPIO_VAL_SRC_B                               1u

#define  CSP_GPIO_NBR_IO_MUX_00                           0u
#define  CSP_GPIO_NBR_IO_MUX_25                          25u
#define  CSP_GPIO_NBR_IO_MUX_41                          41u


/*
*********************************************************************************************************
*                                           LOCAL CONSTANTS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                          LOCAL DATA TYPES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            LOCAL TABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                      LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                     LOCAL CONFIGURATION ERRORS
*********************************************************************************************************
*/



/*
*********************************************************************************************************
*********************************************************************************************************
*                                           LOCAL FUNCTIONS
*********************************************************************************************************
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                          CSP_GPIO_Cfg()
*
* Description : Configure a set of pins for a specific I/O port
*
* Argument(s) : port_nbr    I/O port number. 
*                               CSP_GPIO_PORT_NBR_00            General purpose I/O
*                               CSP_GPIO_PORT_NBR_01            IOMuxes 0  ... 31
*                               CSP_GPIO_PORT_NBR_02            IOMuxes 32 ... 63
*                               CSP_GPIO_PORT_NBR_03            IOMuxes 64 ... 80
*
*               pins        Bit mask specifying which pins to configure.
*
*               dir         Pins direction :
*                               CSP_GPIO_DIR_IN                  Pins are configured as inputs.
*                               CSP_GPIO_DIR_OUT                 Pins are configured as outputs.
*                               CSP_GPIO_DIR_INOUT               Pins are configured as bidirectional pins.  
*
*               drv_mode    Pins Drive mode flags (not used):
*                               CSP_GPIO_FLAG_MODE_NONE          Pins have neither pull-down nor pull-up.
*                               CSP_GPIO_FLAG_MODE_PULLUP        Pins have a pull-up   resistor enabled.
*                               CSP_GPIO_FLAG_MODE_PULLDOWN      Pins have a pull-down resistor enabled.
*                               CSP_GPIO_FLAG_MODE_SCH_TRIGGER   Pins have schmitt trigger.   
*
*               int_en      Pin Interrupt enable:
*                               DEF_YES                          Pin interrupt is enabled.
*                               DEF_NO                           Pin Interrupt is disabled.
*
*               int_pol     Pin interrupt polarity:
*                               CSP_INT_POL_LEVEL_HIGH           Pin interrupt is activated on a high level.
*                               CSP_INT_POL_LEVEL_LOW            Pin interrupt is activated on a low level.
* 
*                               CSP_INT_POL_EDGE_POSITIVE        Pin interrupt is activated on the positive edge.
*                               CSP_INT_POL_EDGE_NEGATIVE        Pin interrupt is activated on the negative edge.
*                               CSP_INT_POL_EDGE_BOTH            Pin interrupt is activated on both edges.       
*             
*               fnct         Pins function:
*                               CSP_GPIO_FNCT_00                 Pins are controlled by the Peripheral in function 0. (see note #2)
*                               CSP_GPIO_FNCT_01                 Pins are controlled by the Peripheral in function 1.
*                               CSP_GPIO_FNCT_02                 Pins are controlled by the Peripheral in function 2.
*                               CSP_GPIO_FNCT_03                 Pins are controlled by the Peripheral in funciton 3.
*
* Return(s)   : DEF_OK    If the pins were configured successfully.
*               DEF_FAIL  If the pins could not be configured successfully.
*
* Caller(s)   : Application.
*
* Note(s)     : (1) The  A2Fx00s Microcontroller subsystem (MSS) contains I/O multiplexers which are involved
*                   in the reuse of some MSS-related I/O pads and in providing a number of options for multiplexing GPIO, 
*                   peripherals signals, and fabric interface signals to the pad.
*                   
*                   The A2Fx00s contains 83 multiplexers, this multiplexers are grouped logically in there ports.
*                               CSP_GPIO_PORT_NBR_1              IOMuxes 0  ... 31
*                               CSP_GPIO_PORT_NBR_2              IOMuxes 32 ... 63
*                               CSP_GPIO_PORT_NBR_3              IOMuxes 64 ... 82
* 
*                   IOMUX model
*
*                                   +---------------+                    
*                                   |      IOMUX    |                    
*                        IN_A  ---> |               |
*                        OUT_A <--- | A port        |<------- IO_1
*                        OE_A  ---> |               |-------- IO_0     
*                                   |               |
*                        IN_B  ---> |               |-------> IO_OE
*                        OUT_B <--- | B port        |
*                        OE_B  ---> |               |
*                                   +---------------+
* 
*                   The 'fnct' argument determines how the signals are routed.
*                             CSP_GPIO_FNCT_00    A <===> IO
*                             CSP_GPIO_FNCT_01    B <===> IO
*                             CSP_GPIO_FNCT_02    A <===> B (excluding the OE signal)                                                          
* 
*               (2) For the A2Fx00s devices, GPIOs 16 trought 31 are multiplexed between IOMUXES. This allows these GPIOs 
*                   acess to user signals form the fabric interface.
*                   
*                                          +----------+
*                    PINS 0 .. 15          |          |
*                   +------------+         | IOMUX_A  |----> GPIO_Pin Fnct0 = 'CSP_GPIO_FNCT_00'
*                   | GPIO PIN x |-------> |          |
*                   +------------+         |          |----> GPIO_Pin Fnct1 = 'CSP_GPIO_FNCT_01'
*                                          +----------+
*
*
*                                          +----------+
*                   PINS 16 .. 31          |          |
*                   +------------+         | IOMUX_A  |----> GPIO_Pin Fnct0 = 'CSP_GPIO_FNCT_00'
*                   | GPIO PIN x |-------> |          |
*                   +------------+   |     |          |----> GPIO_Pin Fnct1 = 'CSP_GPIO_FNCT_01'
*                                    |     +----------+
*                                    |
*                                    |     +----------+
*                                    |     |          |
*                                    ----->| IOMUX_B  |----> GPIO_Pin Fnct2 = 'CSP_GPIO_FNCT_02'
*                                          |          |
*                                          |          |----> GPIO_Pin Fnct3 = 'CSP_GPIO_FNCT_03'
*                                          +----------+
*                                          
*********************************************************************************************************
*/

CPU_BOOLEAN  CSP_GPIO_Cfg (CSP_DEV_NBR    port_nbr,
                           CSP_GPIO_MSK   pins,
                           CSP_OPT        dir,
                           CSP_OPT_FLAGS  drv_mode,
                           CPU_BOOLEAN    int_en,
                           CSP_OPT        int_pol,
                           CSP_OPT        fnct)
{
	CPU_BOOLEAN   gpio_cfg_en;
	CPU_BOOLEAN   gpio_pin_src;
	CPU_INT32U    gpio_cfg_reg;
	CPU_INT08U    io_mux_reg;
	CPU_INT08U    pin_nbr;
    CPU_SR_ALLOC();

                                                                /* ----------- PORT NMBER ARGUMENTS CHECKING ---------- */
    gpio_cfg_en  = DEF_NO;
    gpio_pin_src = CSP_GPIO_VAL_SRC_A;
    io_mux_reg   = DEF_BIT_NONE;
    gpio_cfg_reg = DEF_BIT_NONE;
    
    switch (port_nbr) {
        case CSP_GPIO_PORT_NBR_00:                              /* A2Fx00s devices only have one GPIO port              */
        	 gpio_cfg_en = DEF_YES;
             break;
        
        case CSP_GPIO_PORT_NBR_01:                              /* Fabric signals                                       */             
        case CSP_GPIO_PORT_NBR_02:             
        	 break;
        
        case CSP_GPIO_PORT_NBR_03:
             pins &= DEF_BIT_FIELD(18u, 0u);
             break;
                     
        default:
             return (DEF_FAIL);
    }
                                                                /* ------------- DRIVE MODE CONFIGURATION ------------- */
    if (DEF_BIT_IS_SET(drv_mode, CSP_GPIO_FLAG_MODE_PULLUP)) {
        DEF_BIT_SET(io_mux_reg, CSP_GPIO_BIT_IO_MUX_CFGn_PU);      
    }

    if (DEF_BIT_IS_SET(drv_mode, CSP_GPIO_FLAG_MODE_PULLDOWN)) {
        DEF_BIT_SET(io_mux_reg, CSP_GPIO_BIT_IO_MUX_CFGn_PD);      
    }
    
    if (DEF_BIT_IS_SET(drv_mode, CSP_GPIO_FLAG_MODE_SCH_TRIGGER)) {
        DEF_BIT_SET(io_mux_reg, CSP_GPIO_FLAG_MODE_SCH_TRIGGER);      
    }
    
    if (gpio_cfg_en == DEF_NO) {
        if (dir == CSP_GPIO_DIR_IN) {
            DEF_BIT_SET(io_mux_reg, CSP_GPIO_BIT_IO_MUX_CFGn_M2_S_0);
        } else if (dir == CSP_GPIO_DIR_OUT) {
            DEF_BIT_SET(io_mux_reg, CSP_GPIO_BIT_IO_MUX_CFGn_M2_S_1);
        } else {
            ;
        }
        
        switch (fnct) {                                         /* --------------- IOMUX CONFIGUATION ---------------- */
            case CSP_GPIO_FNCT_00:             
                 DEF_BIT_SET(io_mux_reg, CSP_GPIO_BIT_IO_MUX_CFGn_M2_S_A |
                                         CSP_GPIO_BIT_IO_MUX_CFGn_M1_S_A);
                 break;
    
            case CSP_GPIO_FNCT_01:
                 DEF_BIT_SET(io_mux_reg, CSP_GPIO_BIT_IO_MUX_CFGn_M1_S_B);
                 if (dir == CSP_GPIO_DIR_INOUT) {
                     DEF_BIT_SET(io_mux_reg, CSP_GPIO_BIT_IO_MUX_CFGn_M2_S_B);
                 }                 
                 break;
                
            case CSP_GPIO_FNCT_02:
                 DEF_BIT_SET(io_mux_reg, CSP_GPIO_BIT_IO_MUX_CFGn_M3_S |
                                         CSP_GPIO_BIT_IO_MUX_CFGn_M0_S);
                 break;
           
            default:
                 return (DEF_FAIL);
        }
        
    } else {                                                    /* --------------- GPIO CONFIGURATION ---------------- */
        if (int_en == DEF_YES) {
            switch (int_pol) {
                case CSP_INT_POL_LEVEL_HIGH:
                     DEF_BIT_SET(gpio_cfg_reg, CSP_GPIO_BIT_GPIOCFGn_GPIOINT_HIGH);
                     break;
                     
                case CSP_INT_POL_LEVEL_LOW:
                     DEF_BIT_SET(gpio_cfg_reg, CSP_GPIO_BIT_GPIOCFGn_GPIOINT_LOW);                          
                     break;
                
                case CSP_INT_POL_EDGE_POSITIVE:
                     DEF_BIT_SET(gpio_cfg_reg, CSP_GPIO_BIT_GPIOCFGn_GPIOINT_POS);
                     break;
                              
                 case CSP_INT_POL_EDGE_NEGATIVE:
                      DEF_BIT_SET(gpio_cfg_reg, CSP_GPIO_BIT_GPIOCFGn_GPIOINT_NEG);                          
                      break;

                 case CSP_INT_POL_EDGE_BOTH:
                      DEF_BIT_SET(gpio_cfg_reg, CSP_GPIO_BIT_GPIOCFGn_GPIOINT_BOTH);                         
                      break;
                                                    
                  default:
                      return (DEF_FAIL);
            }
            DEF_BIT_SET(gpio_cfg_reg, CSP_GPIO_BIT_GPIOCFGn_GPINTEN);
        }
            
        if (dir == CSP_GPIO_DIR_IN) {                              
            DEF_BIT_SET(gpio_cfg_reg, CSP_GPIO_BIT_GPIOCFGn_GPINEN);
            DEF_BIT_SET(io_mux_reg, CSP_GPIO_BIT_IO_MUX_CFGn_M2_S_0);
        } else if (dir == CSP_GPIO_DIR_OUT){
            DEF_BIT_SET(gpio_cfg_reg, CSP_GPIO_BIT_GPIOCFGn_GPOUTEN | CSP_GPIO_BIT_GPIOCFGn_GPO_OUTBUFEN);
            DEF_BIT_SET(io_mux_reg, CSP_GPIO_BIT_IO_MUX_CFGn_M2_S_1);
        }
        
        switch (fnct) {
            case CSP_GPIO_FNCT_00:
                 DEF_BIT_SET(io_mux_reg, CSP_GPIO_BIT_IO_MUX_CFGn_M2_S_A |
                                         CSP_GPIO_BIT_IO_MUX_CFGn_M1_S_A);
                 break;
                 
            case CSP_GPIO_FNCT_01:
                 DEF_BIT_SET(io_mux_reg, CSP_GPIO_BIT_IO_MUX_CFGn_M2_S_B |
                                         CSP_GPIO_BIT_IO_MUX_CFGn_M1_S_B);
                 if (dir == CSP_GPIO_DIR_INOUT) {
                     DEF_BIT_SET(io_mux_reg, CSP_GPIO_BIT_IO_MUX_CFGn_M2_S_B);
                 } 
                 break;
                                 
            case CSP_GPIO_FNCT_02:
                 DEF_BIT_SET(io_mux_reg, CSP_GPIO_BIT_IO_MUX_CFGn_M2_S_A |
                                         CSP_GPIO_BIT_IO_MUX_CFGn_M1_S_A);
                 
                 gpio_pin_src = CSP_GPIO_VAL_SRC_B;                 
                 break;
    
            
            case CSP_GPIO_FNCT_03:                
                 DEF_BIT_SET(io_mux_reg, CSP_GPIO_BIT_IO_MUX_CFGn_M2_S_B |
                                         CSP_GPIO_BIT_IO_MUX_CFGn_M1_S_B);
                 gpio_pin_src = CSP_GPIO_VAL_SRC_B;                 
                 break;                 

            default:
                 return (DEF_FAIL);
        }
    }

    for (pin_nbr = 0u; pin_nbr < 32u; pin_nbr++) {
        if (DEF_BIT_IS_SET(pins, DEF_BIT(pin_nbr))) {
            CPU_CRITICAL_ENTER();        
            if (gpio_cfg_en == DEF_YES) {
                CSP_GPIO_REG_GPIOCFGn(pin_nbr) =  gpio_cfg_reg;
                if (pin_nbr > 15u) {
                    if (gpio_pin_src == CSP_GPIO_VAL_SRC_B) {
                        DEF_BIT_SET(CSP_GPIO_REG_GPIN_SRCSEL_CFG, DEF_BIT(pin_nbr - 16u));
                        CSP_GPIO_REG_IO_MUX_CFGn(CSP_GPIO_NBR_IO_MUX_41 + (pin_nbr - 16u)) = io_mux_reg;
                    } else {
                        DEF_BIT_CLR(CSP_GPIO_REG_GPIN_SRCSEL_CFG, DEF_BIT(pin_nbr - 16u));
                        CSP_GPIO_REG_IO_MUX_CFGn(CSP_GPIO_NBR_IO_MUX_00 + (pin_nbr - 16u)) = io_mux_reg;
                    }
                } else {
                    CSP_GPIO_REG_IO_MUX_CFGn(CSP_GPIO_NBR_IO_MUX_25 + pin_nbr) = io_mux_reg;
                }
            } else {
                CSP_GPIO_REG_IO_MUX_CFGn(pin_nbr + ((port_nbr - 1u) * 32u)) = io_mux_reg;
            }
            CPU_CRITICAL_EXIT();
        }
    }
        
    return (DEF_OK);
}


/*
*********************************************************************************************************
*                                         CSP_GPIO_BitClr()
*
* Description : Clear a group of pins for a specific I/O port.
*
* Argument(s) : port_nbr    I/O port number.
*
*               pins        Bit mask specifying which pins to clear.
*
* Return(s)   : none
*
* Caller(s)   : Application.
*
* Note(s)     : none.                   
*********************************************************************************************************
*/

void  CSP_GPIO_BitClr (CSP_DEV_NBR   port_nbr,
                       CSP_GPIO_MSK  pins)
{
    CPU_SR_ALLOC();
    
    
    switch (port_nbr) {
        case CSP_GPIO_PORT_NBR_00:
             break;
             
        default:
             return ;
    }
    
    CPU_CRITICAL_ENTER();
    DEF_BIT_CLR(CSP_GPIO_REG_GPO, pins);
    CPU_CRITICAL_EXIT();
}


/*
*********************************************************************************************************
*                                         CSP_GPIO_BitSet()
*
* Description : Set a group of pins for a specific I/O port
*
* Argument(s) : port_nbr  I/O port number.
*
*               pins      Bit mask specifying which pins to set.
*
* Return(s)   : none
*
* Caller(s)   : Application.
*
* Note(s)     : none.                   
*********************************************************************************************************
*/

void  CSP_GPIO_BitSet (CSP_DEV_NBR   port_nbr,
                       CSP_GPIO_MSK  pins)
{
    CPU_SR_ALLOC();
    
    
    switch (port_nbr) {
        case CSP_GPIO_PORT_NBR_00:
             break;
             
        default:
             return ;
    }
    
    CPU_CRITICAL_ENTER();
    DEF_BIT_SET(CSP_GPIO_REG_GPO, pins);
    CPU_CRITICAL_EXIT();            
}


/*
*********************************************************************************************************
*                                         CSP_GPIO_BitToggle()
*
* Description : Toggle a group of pins for a specific I/O port.
*
* Argument(s) : port_nbr  I/O port number.
*
*               pins      Bit mask specifying which pins to Toggle.
*
* Return(s)   : none
*
* Caller(s)   : Application.
*
* Note(s)     : none.                   
*********************************************************************************************************
*/

void  CSP_GPIO_BitToggle (CSP_DEV_NBR   port_nbr,
                          CSP_GPIO_MSK  pins)                          
{
    CSP_GPIO_MSK  pins_set;
    CSP_GPIO_MSK  pins_clr;
    CPU_SR_ALLOC();
    
    switch (port_nbr) {
        case CSP_GPIO_PORT_NBR_00:
             break;
             
        default:
             return ;
    }
        
    pins_clr =  (CSP_GPIO_REG_GPO) & pins;    
    pins_set = ~(CSP_GPIO_REG_GPO) & pins;

    CPU_CRITICAL_ENTER();
    DEF_BIT_SET(CSP_GPIO_REG_GPO, pins_set);
    DEF_BIT_CLR(CSP_GPIO_REG_GPO, pins_clr);
    CPU_CRITICAL_EXIT();
}   
    

/*
*********************************************************************************************************
*                                         CSP_GPIO_Rd()
*
* Description : Read the current value of the port.
*
* Argument(s) : port_nbr    I/O port number.
*
* Return(s)   : The current value of the port.
*
* Caller(s)   : Application.
*
* Note(s)     : none.                   
*********************************************************************************************************
*/

CSP_GPIO_MSK  CSP_GPIO_Rd (CSP_DEV_NBR  port_nbr)
{
    CSP_GPIO_MSK  port_val;

    
    switch (port_nbr) {
        case CSP_GPIO_PORT_NBR_00:
             break;
             
        default:
             return ((CSP_GPIO_MSK)0);
    }

    port_val  = CSP_GPIO_REG_GPO;
    port_val |= CSP_GPIO_REG_GPI;
    
    return (port_val);
}


/*
*********************************************************************************************************
*                                         CSP_GPIO_Wr()
*
* Description : Write a value to a specific port.
*
* Argument(s) : port    I/O port number.
*
*               val     Value to be written in the port.
*
* Return(s)   : The current value of the port.
*
* Caller(s)   : Application.
*
* Note(s)     : None.
*********************************************************************************************************
*/

void  CSP_GPIO_Wr (CSP_DEV_NBR   port_nbr,
                   CSP_GPIO_MSK  val)
{
    CPU_SR_ALLOC();
    
    
    switch (port_nbr) {
        case CSP_GPIO_PORT_NBR_00:
             break;
             
        default:
             return ;
    }
    
    CPU_CRITICAL_ENTER();
    CSP_GPIO_REG_GPO = val;
    CPU_CRITICAL_EXIT();            
}
