/*
*********************************************************************************************************
*                                             uC/CSP
*                                        CHIP SUPPORT PACKAGE
*
*                          (c) Copyright 2004-2009; Micrium, Inc.; Weston, FL               
*
*               All rights reserved.  Protected by international copyright laws.
*
*               uC/CSP is provided in source form to registered licensees ONLY.  It is 
*               illegal to distribute this source code to any third party unless you receive 
*               written permission by an authorized Micrium representative.  Knowledge of 
*               the source code may NOT be used to develop a similar product.
*
*               Please help us continue to provide the Embedded community with the finest 
*               software available.  Your honesty is greatly appreciated.
*
*               You can contact us at www.micrium.com.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                         uC/CSP FAMILY DEFINES
*                                            ACTEL A2Fx00 
*
* Filename      : CSP_GRP.h
* Version       : V1.00
* Programmer(s) : FT
*********************************************************************************************************
* Note(s)       : (1) The following products are supported by this family definition.
*                    
*                     +==========|===========|==========|=======|==============|
*                     |  DEVICE  | FLASH(KB) | SRAM(KB) | ETHER | SYSTEM GATES |
*                     +==========|===========|==========|======================|
*                     |  A2F200  |    256    |   64     |   Y   |   200,000    |
*                     |  A2F500  |    512    |   64     |   Y   |   500,000    |
*                     +==========|===========|==========|=======|==============|
*
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                               MODULE
*********************************************************************************************************
*/

#ifndef  CSP_GRP_MODULE_PRESENT
#define  CSP_GRP_MODULE_PRESENT


/*
*********************************************************************************************************
*                                            INCLUDE FILES
*********************************************************************************************************
*/

#include <csp_types.h>


/*
*********************************************************************************************************
*                                               EXTERNS
*********************************************************************************************************
*/

#ifdef   CSP_GRP_MODULE
#define  CSP_GRP_EXT
#else
#define  CSP_GRP_EXT  extern
#endif


/*
*********************************************************************************************************
*                                        DEFAULT CONFIGURATION
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                           FAMILY DEFINITION
*********************************************************************************************************
*/

#define  CSP_GRP_ID_A2FX00
#define  CSP_GRP_NAME                     "A2Fx00"


/*
*********************************************************************************************************
*                              INTERRUPT CONTROLLER DEVICE NUMBERS
*********************************************************************************************************
*/

#define  CSP_INT_CTRL_NBR_MAIN                  (CSP_DEV_NBR)(0u)

/*
*********************************************************************************************************
*                                        INTERRUPT SOURCES
*********************************************************************************************************
*/

#define  CSP_INT_SRC_NBR_WDT_00                 (CSP_DEV_NBR)( 0u)    /* Watchdog.                                      */
#define  CSP_INT_SRC_NBR_BROWN_OUT_00           (CSP_DEV_NBR)( 1u)    /* Brown Out 1.5v                                 */
#define  CSP_INT_SRC_NBR_BROWN_OUT_01           (CSP_DEV_NBR)( 2u)    /* Brown Out 3.3v                                 */
#define  CSP_INT_SRC_NBR_RTC_00                 (CSP_DEV_NBR)( 3u)    /* RTC MATCH EVENT                                */
#define  CSP_INT_SRC_NBR_PWR_SUPPLY_00          (CSP_DEV_NBR)( 4u)    /* Power Supply                                   */
#define  CSP_INT_SRC_NBR_ETHER_00               (CSP_DEV_NBR)( 5u)    /* ETHER                                          */
#define  CSP_INT_SRC_NBR_IAP_00                 (CSP_DEV_NBR)( 6u)    /* In-Application Programming                     */
#define  CSP_INT_SRC_NBR_ENVM_00                (CSP_DEV_NBR)( 7u)    /* Embedded non-volatile memory.                  */
#define  CSP_INT_SRC_NBR_ENVM_01                (CSP_DEV_NBR)( 8u)    /* Embedded non-volatile memory.                  */
#define  CSP_INT_SRC_NBR_DMA_00                 (CSP_DEV_NBR)( 9u)    /* DMA                                            */
#define  CSP_INT_SRC_NBR_UART_00                (CSP_DEV_NBR)(10u)    /* UART.                                          */
#define  CSP_INT_SRC_NBR_UART_01                (CSP_DEV_NBR)(11u)    /* UART.                                          */
#define  CSP_INT_SRC_NBR_SPI_00                 (CSP_DEV_NBR)(12u)    /* SPI.                                           */
#define  CSP_INT_SRC_NBR_SPI_01                 (CSP_DEV_NBR)(13u)    /* SPI.                                           */
#define  CSP_INT_SRC_NBR_I2C_00                 (CSP_DEV_NBR)(14u)    /* I2C                                            */
#define  CSP_INT_SRC_NBR_I2C_SMB_ALERT_00       (CSP_DEV_NBR)(15u)    /* I2C SMBUS alert.                               */
#define  CSP_INT_SRC_NBR_I2C_SMB_SUS_00         (CSP_DEV_NBR)(16u)    /* I2C SMBUS suspend.                             */
#define  CSP_INT_SRC_NBR_I2C_01                 (CSP_DEV_NBR)(17u)    /* I2C                                            */
#define  CSP_INT_SRC_NBR_I2C_SMB_ALERT_01       (CSP_DEV_NBR)(18u)    /* I2C SMBUS alert.                               */
#define  CSP_INT_SRC_NBR_I2C_SMB_SUS_01         (CSP_DEV_NBR)(19u)    /* I2C SMBUS suspend.                             */
#define  CSP_INT_SRC_NBR_I2C_TMR_00             (CSP_DEV_NBR)(20u)    /* Timer.                                         */
#define  CSP_INT_SRC_NBR_I2C_TMR_01             (CSP_DEV_NBR)(21u)    /* Timer.                                         */
#define  CSP_INT_SRC_NBR_I2C_PLL_00             (CSP_DEV_NBR)(22u)    /* PLL.                                           */
#define  CSP_INT_SRC_NBR_I2C_PLL_01             (CSP_DEV_NBR)(23u)    /* PLL.                                           */
#define  CSP_INT_SRC_NBR_I2C_AHB_BUS            (CSP_DEV_NBR)(24u)    /* AHBUS                                          */
#define  CSP_INT_SRC_NBR_RSVD_00                (CSP_DEV_NBR)(25u)    /* Reserved.                                      */
#define  CSP_INT_SRC_NBR_RSVD_01                (CSP_DEV_NBR)(26u)    /* Reserved.                                      */
#define  CSP_INT_SRC_NBR_RSVD_02                (CSP_DEV_NBR)(27u)    /* Reserved.                                      */
#define  CSP_INT_SRC_NBR_RSVD_03                (CSP_DEV_NBR)(28u)    /* Reserved.                                      */
#define  CSP_INT_SRC_NBR_RSVD_04                (CSP_DEV_NBR)(29u)    /* Reserved.                                      */
#define  CSP_INT_SRC_NBR_RSVD_05                (CSP_DEV_NBR)(30u)    /* Reserved.                                      */

#define  CSP_INT_SRC_NBR_FAB_IF                 (CSP_DEV_NBR)(31u)    /* Fabric Interface                               */
                                                                      /* GPIO Interrupt                                 */
#define  CSP_INT_SRC_NBR_GPIO_n(pin_nbr)        (CSP_DEV_NBR)(32u + (pin_nbr))
                                                                      /* ACE Interrupt                                 */
#define  CSP_INT_SRC_NBR_ACE_n(int_src)         (CSP_DEV_NBR)(64u + (int_src))

/*
*********************************************************************************************************
*                                     MAXIMUM NUMBER OF DEVICES
*********************************************************************************************************
*/

#define  CSP_NBR_MAX_INT_SRC             (CSP_DEV_NBR)(86u)    /* #Interrupt sources.                                   */


/*
*********************************************************************************************************
*                                               CLK DEFINES
*
* Note(s) : (1) The on-chip RC oscillator funs at nominal frequency of 100Mhz.
*
*           (2) The A2Fx00 defines 3 peripherals clocks.
*
*               PCLK0  Peripheral clock 0 for peripherals on the APB_0 bus.
*               PCLK1  Peripheral clock 1 for peripherals on the APB_1 bus.
*               ACLK   Peripheral clock 2 for peripherals on the APB_2 bus (Analog compute engine).
*               
*********************************************************************************************************
*/

#define  CSP_PM_RC_OS_CLK_FREQ                    100000000u

#define  CSP_PM_PER_CLK_NBR_PCLK_00                       0u
#define  CSP_PM_PER_CLK_NBR_PCLK_01                       1u
#define  CSP_PM_PER_CLK_NBR_ACLK                          2u


/*
*********************************************************************************************************
*                                            DATA TYPES
*********************************************************************************************************
*/

typedef  CPU_INT32U  CSP_GPIO_MSK;                              /* GPIO port size.                                      */
typedef  CPU_INT32U  CSP_TMR_VAL;                               /* Timer value size.                                    */

/*
*********************************************************************************************************
*                                          GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                          GLOBAL VARIABLES
*
* Note(s) (1) The 'CSP_MainVectTbl[]' allows OS to dispatch ISRs and measure Interrupt exceution time.
*********************************************************************************************************
*/
                                                                /* NVIC external interrupts vector table               */
CSP_GRP_EXT  CSP_INT_VECT  CSP_MainVectTbl[CSP_NBR_MAX_INT_SRC]; 


/*
*********************************************************************************************************
*                                              MACRO'S
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                         FUNCTION PROTOTYPES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                        CONFIGURATION ERRORS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                             MODULE END
*********************************************************************************************************
*/

#endif                                                          /* End of CSP_GRP module include.                      */

