/*
*********************************************************************************************************
*                                                uC/CSP
*                                        CHIP SUPPORT PACKAGE
*
*                          (c) Copyright 2004-2009; Micrium, Inc.; Weston, FL               
*
*               All rights reserved.  Protected by international copyright laws.
*
*               uC/CSP is provided in source form to registered licensees ONLY.  It is 
*               illegal to distribute this source code to any third party unless you receive 
*               written permission by an authorized Micrium representative.  Knowledge of 
*               the source code may NOT be used to develop a similar product.
*
*               Please help us continue to provide the Embedded community with the finest 
*               software available.  Your honesty is greatly appreciated.
*
*               You can contact us at www.micrium.com.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                       uC/CSP INTERRUPTS MANAGMENT 
*                                  ACTEL A2Fx00 NVIC EXTERNALS INTERRUPTS
* 
* Filename      : csp_int.c
* Version       : V1.00
* Programmer(s) : FT
*********************************************************************************************************
* Note(s)       : 
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            INCLUDE FILES
*********************************************************************************************************
*/

#define    CSP_INT_MODULE
#include   <csp.h>


/*
*********************************************************************************************************
*                                          DEFAULT CONFIGURATION
*********************************************************************************************************
*/



/*
*********************************************************************************************************
*                                            LOCAL DEFINES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                           LOCAL CONSTANTS
*********************************************************************************************************
*/



/*
*********************************************************************************************************
*                                           LOCAL DATA TYPES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            LOCAL TABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            LOCAL MACRO's
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                      LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                     LOCAL CONFIGURATION ERRORS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*********************************************************************************************************
*                                           LOCAL FUNCTIONS
*********************************************************************************************************
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                          CSP_IntClr()
*
* Description : Clear an Interrupt source on a specific interrupt controller.
*                   
* Argument(s) : int_ctrl   Interrupt controller number.
*                              CSP_INT_CTRL_NBR_MAIN       NVIC interrupt controller.
*
*               src_nbr    Interrupt source number.
*
* Return(s)   : none
*
* Caller(s)   : Application.
*
* Note(s)     : (1) The pending status can be only cleared for the NVIC's external sources.
*********************************************************************************************************
*/

void  CSP_IntClr (CSP_DEV_NBR  int_ctrl,
                  CSP_DEV_NBR  src_nbr)                  
{
    
    if ((int_ctrl == CSP_INT_CTRL_NBR_MAIN) &&                  /* See note #1.                                       */
        (src_nbr   < CSP_NBR_MAX_INT_SRC  )) {
        switch (src_nbr) {
            case CSP_INT_SRC_NBR_RSVD_00:
            case CSP_INT_SRC_NBR_RSVD_01:
            case CSP_INT_SRC_NBR_RSVD_02:
            case CSP_INT_SRC_NBR_RSVD_03:
            case CSP_INT_SRC_NBR_RSVD_04:
            case CSP_INT_SRC_NBR_RSVD_05:
                 break;
                 
            default:
                 CPU_IntSrcPendClr(src_nbr + CPU_INT_EXT0);
                 break;
        }
    }
}


/*
*********************************************************************************************************
*                                          CSP_IntDis()
*
* Description : Disable an interrupt source on a specific interrupt controller.
*                   
* Argument(s) : int_ctrl   Interrupt controller number.
*                              CSP_INT_CTRL_NBR_MAIN       NVIC interrupt controller.
*
*               src_nbr    Interrupt source number.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : (1) Only the NVIC's external sources can be disabled.
*********************************************************************************************************
*/

void  CSP_IntDis (CSP_DEV_NBR  int_ctrl,
                  CSP_DEV_NBR  src_nbr)                   
{   
    if ((int_ctrl == CSP_INT_CTRL_NBR_MAIN) &&                  /* See note #1.                                         */
        (src_nbr   < CSP_NBR_MAX_INT_SRC  )) {
        switch (src_nbr) {
            case CSP_INT_SRC_NBR_RSVD_00:
            case CSP_INT_SRC_NBR_RSVD_01:
            case CSP_INT_SRC_NBR_RSVD_02:
            case CSP_INT_SRC_NBR_RSVD_03:
            case CSP_INT_SRC_NBR_RSVD_04:
            case CSP_INT_SRC_NBR_RSVD_05:
                 break;
                 
            default:
                 CPU_IntSrcDis(src_nbr + CPU_INT_EXT0);
                 break;
        }        
    }
}


/*
*********************************************************************************************************
*                                          CSP_IntEn()
*
* Description : Enable an interrupt source on a specific interrupt controller.
*                   
* Argument(s) : int_ctrl   Interrupt controller number.
*                              CSP_INT_CTRL_NBR_MAIN       NVIC interrupt controller.
*
*               src_nbr    Interrupt source number.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : (1) Only the NVIC's external sources can be enabled.
*********************************************************************************************************
*/


void  CSP_IntEn (CSP_DEV_NBR  int_ctrl,
                 CSP_DEV_NBR  src_nbr)                 
{
    if ((int_ctrl == CSP_INT_CTRL_NBR_MAIN) &&
        (src_nbr   < CSP_NBR_MAX_INT_SRC  )) {
        switch (src_nbr) {
            case CSP_INT_SRC_NBR_RSVD_00:
            case CSP_INT_SRC_NBR_RSVD_01:
            case CSP_INT_SRC_NBR_RSVD_02:
            case CSP_INT_SRC_NBR_RSVD_03:
            case CSP_INT_SRC_NBR_RSVD_04:
            case CSP_INT_SRC_NBR_RSVD_05:
                 break;
                 
            default:
                 CPU_IntSrcEn(src_nbr + CPU_INT_EXT0);
                 break;
        }      
    }
}


/*
*********************************************************************************************************
*                                          CSP_IntInit()
*
* Description : Initiailze the Interrupt controller(s).
*               (1) Initialize the Vector table for external source on the NVIC.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  CSP_IntInit (void)
{
    CPU_INT08U  src_nbr;
    CPU_SR_ALLOC();
    
    for (src_nbr = 0u; src_nbr < CSP_NBR_MAX_INT_SRC; src_nbr++) {
        CPU_IntSrcDis(src_nbr + CPU_INT_EXT0);                  /* Disable external interrupts.                         */

        CPU_CRITICAL_ENTER();                                   /* Initialize the vector table.                         */
        CSP_MainVectTbl[src_nbr].FnctPtr = (CPU_FNCT_PTR )0;
        CSP_MainVectTbl[src_nbr].ArgPtr  = (CPU_VOID    *)0;

#if (CSP_CFG_INT_ISR_EXEC_MEAS_EN == DEF_ENABLED) && \
    (CPU_CFG_TS_EN                == DEF_ENABLED)
        CSP_MainVectTbl[src_nbr].TimeMax = (CPU_TS )0;         /* Intialize Interrupt excecution time.                 */
        CSP_MainVectTbl[src_nbr].TimeCur = (CPU_TS )0;         
#endif        
        CPU_CRITICAL_EXIT();        
    }
}


/*
*********************************************************************************************************
*                                          CSP_IntSrcCfg()
*
* Description : Configure an interrupt source.
*
* Argument(s) : int_ctrl   Interrupt controller number.
*                              CSP_INT_CTRL_NBR_MAIN       NVIC interrupt controller.
*  
*               src_nbr    Interrupt Source number.
*
*               src_prio   Interrupt source priority.
*
*               src_pol    Interrupt source polarity:
*                               CSP_INT_POL_LEVEL_HIGH           Source interrupt is activated on a high level.
*                               CSP_INT_POL_LEVEL_LOW            Source interrupt is activated on a low level. 
*
*                               CSP_INT_POL_EDGE_RISING          Source interrupt is activated on the rising edge.
*                               CSP_INT_POL_EDGE_FALLING         Source interrupt is activated on the falling edge.
*                               CSP_INT_POL_EDGE_BOTH            Source interrupt is activated on both edges.       
*
* Return(s)   : DEF_OK,    if the interrupt source was configured
*               DEF_FAIL,  otherwise.
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/
CPU_BOOLEAN  CSP_IntSrcCfg  (CSP_DEV_NBR     int_ctrl,
                             CSP_DEV_NBR     src_nbr,
                             CSP_OPT         src_prio,
                             CSP_OPT         src_pol)
{
    (void)(src_pol);
    
    if ((int_ctrl == CSP_INT_CTRL_NBR_MAIN  ) &&
        (src_nbr   < CSP_NBR_MAX_INT_SRC    )) {        
        CPU_IntSrcPrioSet((src_nbr + CPU_INT_EXT0), 
                          (src_prio));
        
        return (DEF_OK);
    }
    
    return (DEF_FAIL);
}


/*
*********************************************************************************************************
*                                          CSP_IntVectReg()
*
* Description : Register an interrupt service routine handler for a specific interrupt controller.
*
* Argument(s) : int_ctrl   Interrupt controller number.
*                              CSP_INT_CTRL_NBR_MAIN         NVIC interrupt controller.
*  
*               src_nbr     Interrupt source number.
*
*               isr_fnct    Pointer to the interrupt service routine (ISR).
*
*               p_arg       Pointer to the argument of the interrupt service routine (ISR).
*
* Return(s)   : DEF_OK,    If the vector was registered successfully.
*               DEF_FAIL,  otherwise.
*               
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

CPU_BOOLEAN  CSP_IntVectReg (CSP_DEV_NBR    int_ctrl,
                             CSP_DEV_NBR    src_nbr,
                             CPU_FNCT_PTR   isr_fnct,
                             void          *p_arg)
{
    CPU_BOOLEAN    err;
    CSP_INT_VECT  *p_vect_tbl;
    CPU_SR_ALLOC();
    
    
    err = DEF_FAIL;
    
    if ((int_ctrl == CSP_INT_CTRL_NBR_MAIN) &&
        (src_nbr   < CSP_NBR_MAX_INT_SRC  )) {
        switch (src_nbr) {
            case CSP_INT_SRC_NBR_RSVD_00:
            case CSP_INT_SRC_NBR_RSVD_01:
            case CSP_INT_SRC_NBR_RSVD_02:
            case CSP_INT_SRC_NBR_RSVD_03:
            case CSP_INT_SRC_NBR_RSVD_04:
            case CSP_INT_SRC_NBR_RSVD_05:                 
                 break;
                 
            default:
                 p_vect_tbl = &CSP_MainVectTbl[src_nbr];
                 CPU_CRITICAL_ENTER();
                 p_vect_tbl->FnctPtr = isr_fnct;    
                 p_vect_tbl->ArgPtr  = p_arg;
#if (CSP_CFG_INT_ISR_EXEC_MEAS_EN == DEF_ENABLED) && \
    (CPU_CFG_TS_EN                == DEF_ENABLED)
                 p_vect_tbl->TimeMax = (CPU_TS )0;
                 p_vect_tbl->TimeCur = (CPU_TS )0;         
#endif                         
                 CPU_CRITICAL_EXIT();
                 err = DEF_OK;
                 break;
        }
    }
    
    return (err);
}


/*
*********************************************************************************************************
*                                          CSP_IntVectUnreg()
*
* Description : Un-register an interrupt vector.
*
* Argument(s) : int_ctrl     Interrupt controller
*                              CSP_INT_CTRL_NBR_MAIN         NVIC interrupt controller.
*
*               src_nbr      Interrupt source number.
*
* Return(s)   : DEF_OK,     If the vector was unregistered successfully.
*               DEF_FAIL,   otherwise.
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

CPU_BOOLEAN  CSP_IntVectUnreg (CSP_DEV_NBR  int_ctrl,
                               CSP_DEV_NBR  src_nbr)                         
{
    CPU_BOOLEAN    err;
    CSP_INT_VECT  *p_vect_tbl;    
    CPU_SR_ALLOC();

    err = DEF_OK;

    
    if ((int_ctrl == CSP_INT_CTRL_NBR_MAIN) &&
        (src_nbr   < CSP_NBR_MAX_INT_SRC  )) {
        switch (src_nbr) {
            case CSP_INT_SRC_NBR_RSVD_00:
            case CSP_INT_SRC_NBR_RSVD_01:
            case CSP_INT_SRC_NBR_RSVD_02:
            case CSP_INT_SRC_NBR_RSVD_03:
            case CSP_INT_SRC_NBR_RSVD_04:
            case CSP_INT_SRC_NBR_RSVD_05:                 
                 break;
                 
            default:
                 p_vect_tbl = &CSP_MainVectTbl[src_nbr];    
                 CPU_CRITICAL_ENTER();
                 p_vect_tbl->FnctPtr = (CPU_FNCT_PTR  )0;    
                 p_vect_tbl->ArgPtr  = (CPU_VOID     *)0;
#if (CSP_CFG_INT_ISR_EXEC_MEAS_EN == DEF_ENABLED) && \
    (CPU_CFG_TS_EN                == DEF_ENABLED)
                 p_vect_tbl->TimeMax = (CPU_TS )0;
                 p_vect_tbl->TimeCur = (CPU_TS )0;         
#endif                         
                 CPU_CRITICAL_EXIT();
                 err = DEF_OK;
                 break;
        }
    }

    return (err);
}
