/*
*********************************************************************************************************
*                                                uC/CSP
*                                        CHIP SUPPORT PACKAGE
*
*                          (c) Copyright 2004-2009; Micrium, Inc.; Weston, FL               
*
*               All rights reserved.  Protected by international copyright laws.
*
*               uC/CSP is provided in source form to registered licensees ONLY.  It is 
*               illegal to distribute this source code to any third party unless you receive 
*               written permission by an authorized Micrium representative.  Knowledge of 
*               the source code may NOT be used to develop a similar product.
*
*               Please help us continue to provide the Embedded community with the finest 
*               software available.  Your honesty is greatly appreciated.
*
*               You can contact us at www.micrium.com.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                        POWER & CLOCK MANAGEMENT
*                                           ACTEL A2Fx00
*
* Filename      : csp_pm.c
* Version       : V1.00
* Programmer(s) : FT
*********************************************************************************************************
* Note(s)       : none.
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            INCLUDE FILES
*********************************************************************************************************
*/

#define    CSP_PM_MODULE
#include   <csp.h>

/*
*********************************************************************************************************
*                                            LOCAL DEFINES
*
* Note(s) : (1) The 'GLMUXCFG' & 'GLMUXSEL' field in the Multixplexer configuration register (CCC_MUX_CR)
*               select the output of the 'GLA0' signal that is connected to the FCLK. Tabe 15-21 on the A2Fx00
*               datashhet gives the bit definition for the selected input signal.
*********************************************************************************************************
*/

#define  CSP_PM_ADDR_SYS                        (  (CPU_ADDR   )(0xE0042000))
#define  CSP_PM_REG_CLK_CR                      (* (CPU_REG32 *)(CSP_PM_ADDR_SYS + 0x48))
#define  CSP_PM_REG_CCC_DIV_CR                  (* (CPU_REG32 *)(CSP_PM_ADDR_SYS + 0x4C))
#define  CSP_PM_REG_CCC_MUX_CR                  (* (CPU_REG32 *)(CSP_PM_ADDR_SYS + 0x50))
#define  CSP_PM_REG_CCC_PLL_CR                  (* (CPU_REG32 *)(CSP_PM_ADDR_SYS + 0x54))
#define  CSP_PM_REG_CCC_DLY_CR                  (* (CPU_REG32 *)(CSP_PM_ADDR_SYS + 0x58))
#define  CSP_PM_REG_CCC_SR                      (* (CPU_REG32 *)(CSP_PM_ADDR_SYS + 0x5C))
#define  CSP_PM_REG_OSC_CTRL                    (* (CPU_REG32 *)(CSP_PM_ADDR_SYS + 0x60))


                                                                /* ------ CCC_CLK_CR REGISTER BIT & MASK DEFINES ------ */

#define  CSP_PM_MSK_CLK_CR_PCLK0_DIV           DEF_BIT_FIELD(2u, 2u)
#define  CSP_PM_MSK_CLK_CR_PCLK1_DIV           DEF_BIT_FIELD(2u, 4u)
#define  CSP_PM_MSK_CLK_CR_ACLK_DIV            DEF_BIT_FIELD(6u, 4u)

                                                                /* ------ CCC_MUX_CR REGISTER BIT & MASK DEFINES ------ */

                                                                /* GLMUXCFG/GLMUXSEL mask.                              */
#define  CSP_PM_MSK_CCC_MUX_CR_GLMUX            DEF_BIT_FIELD(4u, 24u)
                                                                /* Bit values for GLA selected input signal             */
#define  CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLA0       DEF_BIT_NONE
#define  CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLA1       DEF_BIT_25
#define  CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLA2       DEF_BIT_26
#define  CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLA3      (DEF_BIT_26 | DEF_BIT_25)
#define  CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLA4      (DEF_BIT_27 | DEF_BIT_26)

                                                                /* Bit values for GLC selected input signal             */
#define  CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLC0       DEF_BIT_24
#define  CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLC1      (DEF_BIT_25 | DEF_BIT_24)
#define  CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLC2       DEF_BIT_27
#define  CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLC3      (DEF_BIT_27 | DEF_BIT_25)
#define  CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLC4      (DEF_BIT_27 | DEF_BIT_26 | DEF_BIT_24)

                                                                /* GLA mux datapath.                                    */
#define  CSP_PM_BIT_CCC_MUX_CR_STATASEL         DEF_BIT_00
#define  CSP_PM_BIT_CCC_MUX_CR_RXASEL           DEF_BIT_01
#define  CSP_PM_BIT_CCC_MUX_CR_DYNASEL          DEF_BIT_02
#define  CSP_PM_MSK_CCC_MUX_CR_SEL_CLKA        (CSP_PM_BIT_CCC_MUX_CR_STATASEL | \
                                                CSP_PM_BIT_CCC_MUX_CR_RXASEL   | \
                                                CSP_PM_BIT_CCC_MUX_CR_DYNASEL)

                                                    
#define  CSP_PM_MSK_CCC_MUX_CR_OAMUX            DEF_BIT_FIELD(3u, 3u)

#define  CSP_PM_BIT_CCC_MUX_CR_OAMUX_BYPASS     DEF_BIT_MASK(0u, 3u)
#define  CSP_PM_BIT_CCC_MUX_CR_OAMUX_CLKA       DEF_BIT_MASK(1u, 3u)
#define  CSP_PM_BIT_CCC_MUX_CR_OAMUX_PLLO_FEED  DEF_BIT_MASK(2u, 3u)
#define  CSP_PM_BIT_CCC_MUX_CR_OAMUX_INVALID    DEF_BIT_MASK(3u, 3u)
#define  CSP_PM_BIT_CCC_MUX_CR_OAMUX_PLLO       DEF_BIT_MASK(4u, 3u)
#define  CSP_PM_BIT_CCC_MUX_CR_OAMUX_PLL90      DEF_BIT_MASK(5u, 3u)
#define  CSP_PM_BIT_CCC_MUX_CR_OAMUX_PLL180     DEF_BIT_MASK(6u, 3u)
#define  CSP_PM_BIT_CCC_MUX_CR_OAMUX_PLL270     DEF_BIT_MASK(7u, 3u)

#define  CSP_PM_BIT_CCC_MUX_CR_BYPASSA          DEF_BIT_06

                                                                /* GLB mux datapath.                                    */
#define  CSP_PM_BIT_CCC_MUX_CR_STATBSEL         DEF_BIT_08
#define  CSP_PM_BIT_CCC_MUX_CR_RXBSEL           DEF_BIT_09
#define  CSP_PM_BIT_CCC_MUX_CR_DYNBSEL          DEF_BIT_10
#define  CSP_PM_MSK_CCC_MUX_CR_SEL_CLKB        (CSP_PM_BIT_CCC_MUX_CR_STATBSEL | \
                                                CSP_PM_BIT_CCC_MUX_CR_RXBSEL   | \
                                                CSP_PM_BIT_CCC_MUX_CR_DYNBSEL)

                                                    
#define  CSP_PM_MSK_CCC_MUX_CR_OBMUX            DEF_BIT_FIELD(3u, 11u)
#define  CSP_PM_BIT_CCC_MUX_CR_OBMUX_BYPASS     DEF_BIT_MASK(0u, 11u)
#define  CSP_PM_BIT_CCC_MUX_CR_OBMUX_CLKB       DEF_BIT_MASK(1u, 11u)
#define  CSP_PM_BIT_CCC_MUX_CR_OBMUX_PLLO_FEED  DEF_BIT_MASK(2u, 11u)
#define  CSP_PM_BIT_CCC_MUX_CR_OBMUX_GLA        DEF_BIT_MASK(3u, 11u)
#define  CSP_PM_BIT_CCC_MUX_CR_OBMUX_PLLO       DEF_BIT_MASK(4u, 11u)
#define  CSP_PM_BIT_CCC_MUX_CR_OBMUX_PLL90      DEF_BIT_MASK(5u, 11u)
#define  CSP_PM_BIT_CCC_MUX_CR_OBMUX_PLL180     DEF_BIT_MASK(6u, 11u)
#define  CSP_PM_BIT_CCC_MUX_CR_OBMUX_PLL270     DEF_BIT_MASK(7u, 11u)

#define  CSP_PM_BIT_CCC_MUX_CR_BYPASSB          DEF_BIT_14

                                                                /* GLB mux datapath.                                    */
#define  CSP_PM_BIT_CCC_MUX_CR_STATCSEL         DEF_BIT_16
#define  CSP_PM_BIT_CCC_MUX_CR_RXCSEL           DEF_BIT_17
#define  CSP_PM_BIT_CCC_MUX_CR_DYNCSEL          DEF_BIT_18
#define  CSP_PM_MSK_CCC_MUX_CR_SEL_CLKC        (CSP_PM_BIT_CCC_MUX_CR_STATBSEL | \
                                                CSP_PM_BIT_CCC_MUX_CR_RXCSEL   | \
                                                CSP_PM_BIT_CCC_MUX_CR_DYNCSEL)

                                                    
#define  CSP_PM_MSK_CCC_MUX_CR_OCMUX            DEF_BIT_FIELD(3u, 19u)
#define  CSP_PM_BIT_CCC_MUX_CR_OCMUX_BYPASS     DEF_BIT_MASK(0u, 19u)
#define  CSP_PM_BIT_CCC_MUX_CR_OCMUX_CLKC       DEF_BIT_MASK(1u, 19u)
#define  CSP_PM_BIT_CCC_MUX_CR_OCMUX_PLLO_FEED  DEF_BIT_MASK(2u, 19u)
#define  CSP_PM_BIT_CCC_MUX_CR_OCMUX_GLA        DEF_BIT_MASK(3u, 19u)
#define  CSP_PM_BIT_CCC_MUX_CR_OCMUX_PLLO       DEF_BIT_MASK(4u, 19u)
#define  CSP_PM_BIT_CCC_MUX_CR_OCMUX_PLL90      DEF_BIT_MASK(5u, 19u)
#define  CSP_PM_BIT_CCC_MUX_CR_OCMUX_PLL180     DEF_BIT_MASK(6u, 19u)
#define  CSP_PM_BIT_CCC_MUX_CR_OCMUX_PLL270     DEF_BIT_MASK(7u, 19u)

#define  CSP_PM_BIT_CCC_MUX_CR_BYPASSC          DEF_BIT_22
                                                                /* ------- CCC_DIV_CR REGISTER BIT & MASK DEFINES ----- */                                                               
                                                                /* GLA dividers datapath.                               */
#define  CSP_PM_MSK_CCC_DIV_CR_OADIV            DEF_BIT_FIELD(5u, 0u)
#define  CSP_PM_BIT_CCC_DIV_CR_OADIVHALF        DEF_BIT_05

                                                                /* GLB dividers datapath.                               */
#define  CSP_PM_MSK_CCC_DIV_CR_OBDIV            DEF_BIT_FIELD(5u, 8u)
#define  CSP_PM_BIT_CCC_DIV_CR_OBDIVHALF        DEF_BIT_13

                                                                /* GLC dividers datapath.                               */
#define  CSP_PM_MSK_CCC_DIV_CR_OCDIV            DEF_BIT_FIELD(5u, 16u)
#define  CSP_PM_BIT_CCC_DIV_CR_OCDIVHALF        DEF_BIT_21

                                                                /* -------- CCC_SR REGISTER BIT & MASK DEFINES -------- */
#define  CSP_PM_BIT_CCC_SR_PLL_LOCK             DEF_BIT_00

                                                                /* ------ CCC_PLL_CR REGISTER BIT & MASK DEFINES ------ */
#define  CSP_PM_MSK_CCC_PLL_CR_FBDIV            DEF_BIT_FIELD(7u, 7u)
#define  CSP_PM_MSK_CCC_PLL_CR_FINDIV           DEF_BIT_FIELD(7u, 0u)

#define  CSP_PM_VAL_CCC_CLK_SRC_A                         0u
#define  CSP_PM_VAL_CCC_CLK_SRC_B                         1u
#define  CSP_PM_VAL_CCC_CLK_SRC_C                         2u


/*
*********************************************************************************************************
*                                           LOCAL CONSTANTS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                          LOCAL DATA TYPES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            LOCAL TABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            LOCAL MACRO's
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                      LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/

CPU_INT32U  CSP_PM_PLL_FreqGet (void);

/*
*********************************************************************************************************
*                                     LOCAL CONFIGURATION ERRORS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*********************************************************************************************************
*                                           LOCAL FUNCTIONS
*********************************************************************************************************
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                         CSP_PM_CPU_ClkFreqGet()
*
* Description : Return the CPU clock frequency.
*                   
* Argument(s) : none
*  
* Return(s)   : The CPU frequency in Hertz.
*
* Caller(s)   : Application.
*
* Note(s)     : (1) This diagram describe the SmartFusion clocking scheme:
*
*                                Clock Conditioning 
*                                   Circuitry                    +-------------------------+
*                                 +-----------+  GLA1            |                         |
*                   32Khz   ----> |           |----------------->|                         |
*                                 |           |  GLA0            |                         |
*                 Main Osc  ----> |  MSS_CCC  |----------X------>|    FPGA FABRIC LOGIC    |
*                                 |           |  GLB     |       |                         |
*                  RC Osc   ----> |           |----------|------>|                         |
*                                 |           |  GLC     |       |                         |
*                                 |           |--X-------|------>|                         |
*                                 +-----------+  |       |       +-------------------------+
*                                                |       |     +---------------------------+
*                                                |       |     | Microcontroller Subsystem |
*                                                |       |     |                           |
*                                                |       |---> | FCLK                      |
*                                                |     |\      |                           |
*                                                |     | |     |                           |
*                  External                      |---> | |---> | 10/100 CLK                |
*                  10/100 Clock   -------------------> | |     +---------------------------+
*                                                      |/
*
*                  where  32Khz    is the slow lock oscillator (configured via 'CSP_CFG_PM_SLOW_XTAL_FREQ_HZ')
*                         Main OSC is the main oscillator      (configured via 'CSP_CFG_PM_MAIN_XTAL_FREQ_HZ')
*                         RC   OSC is the internal 100 Mhz oscillator.
*
*                (2) The input clock selection selects the following signals:
*                        AUIN/BUIN/CUIN
*                        AUIP/BUIP/CUIP
*                        ADIP/BDIP/CDIP
*                        GLAINT/GLBINT/GLCINT
*                        RC Oscillator
*                        MAin Oscillator
*
*                    Only the RC and Main Ooscillator cases are managed in this function.
*                    for the other cases this function will return 00'
*********************************************************************************************************
*/

CPU_INT32U  CSP_PM_CPU_ClkFreqGet (void)
{
    CPU_INT32U   reg_div;
    CPU_INT32U   reg_mux;    
    CPU_INT08U   clk_src; 
    CPU_INT32U   clk_src_freq;
    CPU_INT08U   clk_div;
    CPU_INT32U   cpu_freq; 
    
    
    reg_mux     = CSP_PM_REG_CCC_MUX_CR;
    reg_div     = CSP_PM_REG_CCC_DIV_CR;
    cpu_freq    = 0u;
    
                                                                /* Determine the source datapath ....                   */
    switch (reg_mux & CSP_PM_MSK_CCC_MUX_CR_GLMUX) {
        case CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLA0:
        case CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLA1:
        case CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLA2:
        case CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLA3:
        case CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLA4:        
             clk_src = CSP_PM_VAL_CCC_CLK_SRC_A;                /* ... GLA0 comes from CLKA datapath.                   */
             break;
             
        case CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLC0:
        case CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLC1:
        case CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLC2:
        case CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLC3:
        case CSP_PM_BIT_CCC_MUX_CR_GLMUX_GLC4:
             clk_src = CSP_PM_VAL_CCC_CLK_SRC_C;                /* ... GLA1 comes from CLKB datapath                    */                          
             break;                             
             
        default:
             return (0u);                                       /* ... invalid configuration.                           */
    }
                                                                /* Calculate the CLKA datapath if :                     */
                                                                /* ... CLKA is the source or ...                        */
                                                                /* ... CLKC is the source and is not bypassed and ...   */
                                                                /* ... CLKC mux select the GLA clock.                   */

    if ( (clk_src                                                == CSP_PM_VAL_CCC_CLK_SRC_A       ) ||
        ((DEF_BIT_IS_SET(reg_mux, CSP_PM_BIT_CCC_MUX_CR_BYPASSA) == DEF_YES                        ) &&
        ((reg_mux & CSP_PM_MSK_CCC_MUX_CR_OCMUX)                 == CSP_PM_BIT_CCC_MUX_CR_OCMUX_GLA))) {
         
                                                                /* ... determine the CLKA source.                       */
        if (DEF_BIT_IS_SET(reg_mux, CSP_PM_BIT_CCC_MUX_CR_RXASEL)) {
            if (DEF_BIT_IS_CLR(reg_mux, CSP_PM_BIT_CCC_MUX_CR_DYNASEL)) {
                clk_src_freq = CSP_PM_RC_OS_CLK_FREQ;           /* Internal RC oscillator.                              */
            } else {
                clk_src_freq = CSP_CFG_PM_MAIN_XTAL_FREQ_HZ;    /* External Main oscillaotr.                            */
            }
        } else {
            return (0u);                                        /* $$$$ (see note #2)                                   */          
        }
        
        
        if (DEF_BIT_IS_CLR(reg_mux, CSP_PM_BIT_CCC_MUX_CR_BYPASSA)) {
            clk_div     = (reg_div & CSP_PM_MSK_CCC_DIV_CR_OADIV);
            clk_div++;
            
            switch (reg_mux & CSP_PM_MSK_CCC_MUX_CR_OAMUX) {
                case CSP_PM_BIT_CCC_MUX_CR_OAMUX_BYPASS:                      
                case CSP_PM_BIT_CCC_MUX_CR_OAMUX_CLKA:  
                     cpu_freq = clk_src_freq;
                     if (DEF_BIT_IS_SET(reg_div, CSP_PM_BIT_CCC_DIV_CR_OADIVHALF)) {
                         if (clk_div != 1u) {
                             cpu_freq = (cpu_freq * 2u ) / clk_div;
                         }
                         
                     } else {
                         cpu_freq /= clk_div;
                     }
                     break;
                         
                case CSP_PM_BIT_CCC_MUX_CR_OAMUX_PLLO_FEED:
                case CSP_PM_BIT_CCC_MUX_CR_OAMUX_PLLO:
                case CSP_PM_BIT_CCC_MUX_CR_OAMUX_PLL90:
                case CSP_PM_BIT_CCC_MUX_CR_OAMUX_PLL180:
                case CSP_PM_BIT_CCC_MUX_CR_OAMUX_PLL270:                
                     cpu_freq = CSP_PM_PLL_FreqGet();\
                     cpu_freq /= clk_div;                         
                     break;
                
                default:
                     return (0u);
            }
        } else {
            cpu_freq = clk_src_freq;
        }
     }
 
     if (clk_src == CSP_PM_VAL_CCC_CLK_SRC_C) {
        if (DEF_BIT_IS_SET(reg_mux, CSP_PM_BIT_CCC_MUX_CR_RXCSEL)) {
            if (DEF_BIT_IS_CLR(reg_mux, CSP_PM_BIT_CCC_MUX_CR_DYNCSEL)) {
                clk_src_freq = CSP_PM_RC_OS_CLK_FREQ;           /* Internal RC oscillator.                              */
            } else {
                clk_src_freq = CSP_CFG_PM_SLOW_XTAL_FREQ_HZ;    /* External 32Khz oscillaotr.                           */
            }
        } else {
            return (0u);                                        /* $$$$ (see note #2)                                   */
            
        }
                
        if (DEF_BIT_IS_CLR(reg_mux, CSP_PM_BIT_CCC_MUX_CR_BYPASSC)) {
            clk_div     = (reg_div & CSP_PM_MSK_CCC_DIV_CR_OCDIV) >> 16u;
            clk_div++;
            
            switch (reg_mux & CSP_PM_MSK_CCC_MUX_CR_OCMUX) {
                case CSP_PM_BIT_CCC_MUX_CR_OCMUX_BYPASS:                      
                case CSP_PM_BIT_CCC_MUX_CR_OCMUX_CLKC:  
                     cpu_freq = clk_src_freq;
                     if (DEF_BIT_IS_SET(reg_div, CSP_PM_BIT_CCC_DIV_CR_OCDIVHALF)) {
                         if (clk_div != 1u) {
                             cpu_freq = (cpu_freq * 2u ) / clk_div;
                         }
                         
                     } else {
                         cpu_freq /= clk_div;
                     }
                     break;
                         
                case CSP_PM_BIT_CCC_MUX_CR_OCMUX_PLLO_FEED:
                case CSP_PM_BIT_CCC_MUX_CR_OCMUX_PLLO:
                case CSP_PM_BIT_CCC_MUX_CR_OCMUX_PLL90:
                case CSP_PM_BIT_CCC_MUX_CR_OCMUX_PLL180:
                case CSP_PM_BIT_CCC_MUX_CR_OCMUX_PLL270:                
                     cpu_freq = CSP_PM_PLL_FreqGet();\
                     cpu_freq /= clk_div;                         
                     break;

                case CSP_PM_BIT_CCC_MUX_CR_OCMUX_GLA:
                     break;
                
                default:
                     return (0u);
            }
        } else {
            cpu_freq = clk_src_freq;
        }    
    }

    return (cpu_freq);
}


/*
*********************************************************************************************************
*                                           CSP_PM_PLL_FreqGet ()
*
* Description : Get the PLL frequency
*                   
* Argument(s) : none.
*  
*
* Return(s)   : The PLL frequency. If the PLL is disabled '0' will be returned.
*
* Caller(s)   : CSP_PM_CPU_ClkFreqGet()
*
* Note(s)     : none.
*********************************************************************************************************
*/

CPU_INT32U  CSP_PM_PLL_FreqGet (void)
{
    CPU_INT32U  pll_freq;
    CPU_INT32U  pll_m;
    CPU_INT32U  pll_n;
    CPU_INT32U  reg_val;
    
    
    if (DEF_BIT_IS_CLR(CSP_PM_REG_CCC_SR, CSP_PM_BIT_CCC_SR_PLL_LOCK)) {
        return 0;
    }

    reg_val = CSP_PM_REG_CCC_MUX_CR;

    if (DEF_BIT_IS_SET(reg_val, CSP_PM_BIT_CCC_MUX_CR_RXASEL)) {
        if (DEF_BIT_IS_CLR(reg_val, CSP_PM_BIT_CCC_MUX_CR_DYNASEL)) {
            pll_freq = CSP_PM_RC_OS_CLK_FREQ;                   /* Internal RC oscillator.                              */
        } else {
            pll_freq = CSP_CFG_PM_MAIN_XTAL_FREQ_HZ;            /* External Main oscillaotr.                            */
        }
    } else {
        return (0u);                                            /* $$$$ (see 'CSP_PM_CPU_ClkFreqGet()' note #2)         */          
    }
    
    reg_val = CSP_PM_REG_CCC_PLL_CR;
    
    pll_m    = ((reg_val & CSP_PM_MSK_CCC_PLL_CR_FBDIV ) >> 7u) + 1u;
    pll_n    = ((reg_val & CSP_PM_MSK_CCC_PLL_CR_FINDIV)      ) + 1u;
    
    pll_freq = (pll_freq * pll_m) / pll_n;
    
    return (pll_freq);
}


/*
*********************************************************************************************************
*                                           CSP_PM_PerClkEn()
*
* Description : Enable a peripheral clock.
*                   
* Argument(s) : clk_nbr     Peripheral clock id.
*  
*               clk_div     Peripheral Clock divider.
*                               CSP_PM_PER_CLK_DIV_1  Peripheral clock is CCLK.
*                               CSP_PM_PER_CLK_DIV_2  Peripheral clock is CCLK/2.
*                               CSP_PM_PER_CLK_DIV_4  Peripheral clock is CCLK/4.
*                               CSP_PM_PER_CLK_DIV_6  Peripheral clock is CCLK/6 (see note #1).
*                               CSP_PM_PER_CLK_DIV_8  Peripheral clock is CCLK/8 (see note #1).
*
* Return(s)   : DEF_OK   If the clock was configured.
*               DEF_FAIL If the clock could not be configured.
*
* Caller(s)   : Application.
*
* Note(s)     : (1) The CSP_PM_PER_CLK_DIV_6 divider can be only used with the following clks:
*                             CSP_PM_PER_CLK_NBR_CAN1          
*                             CSP_PM_PER_CLK_NBR_CAN2          
*                             CSP_PM_PER_CLK_NBR_ACF              
*********************************************************************************************************
*/

CPU_BOOLEAN  CSP_PM_PerClkCfg (CSP_DEV_NBR  clk_nbr,
                               CSP_OPT      clk_div)
{
    (void)&clk_nbr;
    (void)&clk_div;
    
    return (DEF_OK);
}    


/*
*********************************************************************************************************
*                                         CSP_PM_PerClkEn()
*
* Description : Enable a peripheral clock.
*                   
* Argument(s) : clk_nbr     Peripheral clock id.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : (1) The Peripheral clocks CSP_PM_PER_CLK_NBR_PCLK_00, CSP_PM_PER_CLK_NBR_PCLK_01 & 
*                   CSP_PM_PER_CLK_NBR_ACLK cannot be enabled.
*********************************************************************************************************
*/

void  CSP_PM_PerClkEn (CSP_DEV_NBR clk_nbr)
{
    (void)&clk_nbr;
}


/*
*********************************************************************************************************
*                                         CSP_PM_PerClkDis()
*
* Description : Disable a peripheral clock.
*                   
* Argument(s) : clk_nbr   Peripheral clock id.
*
* Return(s)   : none
*
* Caller(s)   : Application.
*
* Note(s)     : (1) The Peripheral clocks 'CSP_PM_PER_CLK_NBR_PCLK_00', 'CSP_PM_PER_CLK_NBR_PCLK_01' & 
*                   'CSP_PM_PER_CLK_NBR_ACLK' cannot be disabled.
*********************************************************************************************************
*/

void  CSP_PM_PerClkDis (CSP_DEV_NBR  clk_nbr)
{
    (void)&clk_nbr;
}


/*
*********************************************************************************************************
*                                        CSP_PM_PerClkFreqGet()
*
* Description : Return the Peripheral clock
*                   
* Argument(s) : clk_nbr     peripheral clock id.
*                               CSP_PM_PER_CLK_NBR_PCLK_00   Peripherals clock on 'APB_BUS0' bus.
*                               CSP_PM_PER_CLK_NBR_PCLK_01   Peripherals clock on 'APB_BUS1' bus.
*                               CSP_PM_PER_CLK_NBR_ACLK      Peripherals clock on 'APB_BUS2' bus.
*
* Return(s)   : The peripheral clock frequency.
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

CPU_INT32U  CSP_PM_PerClkFreqGet (CSP_DEV_NBR  clk_nbr)
{
    CPU_INT32U  per_clk_freq;
    CPU_INT08U  clk_div;
    
    
    per_clk_freq = CSP_PM_CPU_ClkFreqGet();
    switch (clk_nbr) {
        case CSP_PM_PER_CLK_NBR_PCLK_00:
             clk_div = ((CSP_PM_REG_CLK_CR & CSP_PM_MSK_CLK_CR_PCLK0_DIV) >> 2u);
             break;
             
        case CSP_PM_PER_CLK_NBR_PCLK_01:
             clk_div = ((CSP_PM_REG_CLK_CR & CSP_PM_MSK_CLK_CR_PCLK1_DIV) >> 4u);
             break;
        
        case CSP_PM_PER_CLK_NBR_ACLK:
             clk_div = ((CSP_PM_REG_CLK_CR & CSP_PM_MSK_CLK_CR_ACLK_DIV) >> 6u);
             break;
             
        default:
             return (0u);
    }
    
    if (clk_div == 3u) {
        return 0;
    }
    clk_div       = DEF_BIT(clk_div);    
    per_clk_freq /= clk_div;
    
    return (per_clk_freq);
}


/*
*********************************************************************************************************
*                                        CSP_PM_SysClkEn()
*
* Description : Enable a System Clock.
*                   
* Argument(s) : clk_nbr     System Clock number.  
*                         
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : (1) The current implementation of the CSP for the A2Fx00 does NOT define ANY system clock.
*********************************************************************************************************
*/

void  CSP_PM_SysClkEn (CSP_DEV_NBR  clk_nbr)
{
    (void)clk_nbr;
}


/*
*********************************************************************************************************
*                                        CSP_PM_SysClkDis()
*
* Description : Disable a System Clock
*                   
* Argument(s) : clk_nbr     System Clock number.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : (1) See note 'CSP_PM_SysClkEn()' note #1.
*********************************************************************************************************
*/

void  CSP_PM_SysClkDis (CSP_DEV_NBR clk_nbr)
{
    (void)clk_nbr;
}


/*
*********************************************************************************************************
*                                        CSP_PM_SysClkFreqGet()
*
* Description : Return the frequency for a specific system clock.
*                   
* Argument(s) : clk_nbr     System Clock number
*
* Return(s)   : The system clock frequency.
*
* Caller(s)   : Application.
*
* Note(s)     : (1) See note 'CSP_PM_SysClkEn()' note #1.
*********************************************************************************************************
*/

CPU_INT32U  CSP_PM_SysClkFreqGet (CSP_DEV_NBR  clk_nbr)
{
    (void)clk_nbr;
    
    return ((CPU_INT32U)0);
}
