/*
*********************************************************************************************************
*                                                uC/CSP
*                                        CHIP SUPPORT PACKAGE
*
*                          (c) Copyright 2004-2009; Micrium, Inc.; Weston, FL               
*
*               All rights reserved.  Protected by international copyright laws.
*
*               uC/CSP is provided in source form to registered licensees ONLY.  It is 
*               illegal to distribute this source code to any third party unless you receive 
*               written permission by an authorized Micrium representative.  Knowledge of 
*               the source code may NOT be used to develop a similar product.
*
*               Please help us continue to provide the Embedded community with the finest 
*               software available.  Your honesty is greatly appreciated.
*
*               You can contact us at www.micrium.com.
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*
*                                            TIMER MANAGEMENT
*                                         SMARTFUSION ACTEL A2Fx00
* 
* Filename      : csp_tmr.h
* Version       : V1.00
* Programmer(s) : FT
*********************************************************************************************************
* Note(s)       : (1) $$$$ Timer management has not been developed for the A2Fx00 family
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            INCLUDE FILES
*********************************************************************************************************
*/

#define  CSP_TMR_MODULE
#include <csp.h>

/*
*********************************************************************************************************
*                                    REGISTER & BIT-FIELD DEFINITION
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                           LOCAL CONSTANTS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                          LOCAL DATA TYPES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            LOCAL TABLES
*********************************************************************************************************
*/



/*
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            LOCAL MACRO's
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                      LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                     LOCAL CONFIGURATION ERRORS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*********************************************************************************************************
*                                           LOCAL FUNCTIONS
*********************************************************************************************************
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                             CSP_TmrCfg()
*
* Description : Configure a specific periodic/free running timer 
*                   
* Argument(s) : tmr_nbr    Timer number identifier.
*                              CSP_TMR_NBR_00
*                              CSP_TMR_NBR_01
*                              CSP_TMR_NBR_02
*                              CSP_TMR_NBR_03
*
*               freq       Frequency of the periodic timer.
*                          
*                          If 'freq' == 0 the timer is configured as free-running timer
*                          otherwise will be configured to generate a interrupt event specified by
*                          the frequency of the timer.                          
*
* Return(s)   : none
*
* Caller(s)   : Application.
*
* Note(s)     : (1) The match channel 0 is always used for periodic events.
*
*               (2) If the timer is configured to be used in periodic mode the interrupt handler needs
*                   to be installed first in the interrupt controller and the interrupt needs to be enabled
*                   before calling this function.
*********************************************************************************************************
*/

CPU_BOOLEAN  CSP_TmrCfg (CSP_DEV_NBR  tmr_nbr,
                         CPU_INT32U   freq)
{
    (void)&tmr_nbr;
    (void)&freq;
    return (DEF_OK);
}


/*
*********************************************************************************************************
*                                        CSP_TmrIntClr()
*
* Description : Clear a periodic timer interrupt.
*                   
* Argument(s) : tmr_nbr    Timer number identifier.
*                              CSP_TMR_NBR_00
*                              CSP_TMR_NBR_01
*                              CSP_TMR_NBR_02
*                              CSP_TMR_NBR_03
*
* Return(s)   : none
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  CSP_TmrIntClr (CSP_DEV_NBR tmr_nbr)
{
    (void)&tmr_nbr;
}

/*
*********************************************************************************************************
*                                        CSP_TmrRst()
*
* Description : Resets a timer
*                   
* Argument(s) : tmr_nbr    Timer number identifier.
*                              CSP_TMR_NBR_00
*                              CSP_TMR_NBR_01
*                              CSP_TMR_NBR_02
*                              CSP_TMR_NBR_03
*
* Return(s)   : none
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  CSP_TmrRst (CSP_DEV_NBR  tmr_nbr)
{
    (void)&tmr_nbr;
}


/*
*********************************************************************************************************
*                                          CSP_TmrRd()
*
* Description : Read the current value of a timer.
*                   
* Argument(s) : tmr_nbr    Timer number identifier.
*                              CSP_TMR_NBR_00
*                              CSP_TMR_NBR_01
*                              CSP_TMR_NBR_02
*                              CSP_TMR_NBR_03
*
* Return(s)   : The current value of the timer.
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

CSP_TMR_VAL  CSP_TmrRd  (CSP_DEV_NBR  tmr_nbr)
{
    (void)&tmr_nbr;
    
    return (CSP_TMR_VAL)0;
}

/*
*********************************************************************************************************
*                                     CSP_TmrStart()
*
* Description : Start a timer.
*                   
* Argument(s) : tmr_nbr    Timer number identifier.
*                              CSP_TMR_NBR_00
*                              CSP_TMR_NBR_01
*                              CSP_TMR_NBR_02
*                              CSP_TMR_NBR_03
*
* Return(s)   : none
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  CSP_TmrStart (CSP_DEV_NBR  tmr_nbr)
{
    (void)&tmr_nbr;
}


/*
*********************************************************************************************************
*                                        CSP_TmrStop()
*
* Description : Stop a Timer.
*                   
* Argument(s) : tmr_nbr    Timer number identifier.
*                              CSP_TMR_NBR_00
*                              CSP_TMR_NBR_01
*                              CSP_TMR_NBR_02
*                              CSP_TMR_NBR_03
* Return(s)   : none
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  CSP_TmrStop  (CSP_DEV_NBR  tmr_nbr)
{
    (void)&tmr_nbr;
}

/*
*********************************************************************************************************
*                                         CSP_TmrWr()
*
* Description : Write a value to the timer.
*                   
* Argument(s) : tmr_nbr    Timer number identifier.
*                              CSP_TMR_NBR_00
*                              CSP_TMR_NBR_01
*                              CSP_TMR_NBR_02
*                              CSP_TMR_NBR_03
*
*               tmr_val    Timer Value to be written.
*
* Return(s)   : none
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  CSP_TmrWr  (CSP_DEV_NBR  tmr_nbr,
                  CSP_TMR_VAL  tmr_val)
{
    (void)&tmr_nbr;
}
