extern "C"
{
#include "MessageProcessor.h"
#include "SerialInterruptFake.fff.h"
#include "uCosII_TestDouble.fff.h"
}
#include <string.h>
#include <stddef.h>


#include "CppUTest/TestHarness.h"

static const char * fakeInput;
static char buf[100];
void Successful_OSSemPend(OS_EVENT *, INT32U, INT8U *error)
{
    memcpy(&buf[0], fakeInput, strlen(fakeInput));
    *error = OS_ERR_NONE;
}

void Timeout_OSSemPend(OS_EVENT *, INT32U, INT8U *error)
{
    *error = OS_ERR_TIMEOUT;
}

void TimeOut_OSSemPend(OS_EVENT *, INT32U, INT8U *error)
{
    *error = OS_ERR_TIMEOUT;
}

TEST_GROUP(MessageProcessor)
{
    int result;
    size_t length;
    char * buffer;

    void setup()
    {
        FFF_RESET;
       OSSemPend_fake.custom_fake = Successful_OSSemPend;
        length = sizeof(buffer);
        buffer = reinterpret_cast<char*>(&buf);
        MessageProcessor_Create();
    }

    void teardown()
    {
       MessageProcessor_Destroy();
    }

    void CheckSerialInterruptCalledWith(char * buffer, size_t length)
    {
        LONGS_EQUAL(1, SerialInterrupt_WaitingFor_fake.call_count);
        POINTERS_EQUAL(buffer, SerialInterrupt_WaitingFor_fake.arg0_val);
        LONGS_EQUAL(length, SerialInterrupt_WaitingFor_fake.arg1_val);
    }

        void CheckOSSPendCall()
        {
            LONGS_EQUAL(1, OSSemPend_fake.call_count);
            POINTERS_EQUAL(SerialInterrupt_WaitingFor_fake.arg2_val,
                    OSSemPend_fake.arg0_val);
        }

        void CheckOutputBufferContains(const char * expected)
        {
            STRCMP_EQUAL(expected, buffer);
        }
};

#ifdef refactored_below
TEST(MessageProcessor, WaitForMessage_succeeds)
{
    OSSemPend_reset();
    SerialInterrupt_WaitingFor_reset();
    OSSemPend_fake.custom_fake = Successful_OSSemPend;
    fakeInput = "sched lightOn 5 Monday 20:00";
    CHECK(MessageProcessor_WaitForMessage(
            (char*)&buffer, sizeof(buffer)));
    LONGS_EQUAL(1, SerialInterrupt_WaitingFor_fake.call_count);
    POINTERS_EQUAL(&buffer, SerialInterrupt_WaitingFor_fake.arg0_val);
    LONGS_EQUAL(sizeof(buffer),
            SerialInterrupt_WaitingFor_fake.arg1_val);
    LONGS_EQUAL(1, OSSemPend_fake.call_count);
    POINTERS_EQUAL(SerialInterrupt_WaitingFor_fake.arg2_val,
            OSSemPend_fake.arg0_val);
    STRCMP_EQUAL(fakeInput, buffer);
}
#endif

TEST(MessageProcessor, WaitForMessage_succeeds)
{
    fakeInput = "sched light-on 5 Monday 20:00";
    result = MessageProcessor_WaitForMessage(buffer, length);
    CHECK_TRUE(result);
    CheckSerialInterruptCalledWith(buffer, length);
    CheckOSSPendCall();
    CheckOutputBufferContains(fakeInput);
}

TEST(MessageProcessor, WaitForMessage_timeout)
{
    OSSemPend_fake.custom_fake = TimeOut_OSSemPend;
    strncpy(buffer, "some leftover garbage", length);
    result = MessageProcessor_WaitForMessage(buffer, length);
    CHECK_FALSE(result);
    CheckOutputBufferContains("");
}

